-----------------------------------------------------------------------------------------------
-- Client Lua Script for ValuePlates
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
--[[ List of things to do:
		Do some test runs, super close to being done! Blinds and stealthing will be an obvious issue. 
		Reward icons not automatically repositioning properly in some cases
--]]

require "Window"
require "ChallengesLib"
require "Unit"
require "GameLib"
require "Apollo"
require "PathMission"
require "Quest"
require "Episode"
require "math"
require "string"
require "DialogSys"
require "PublicEvent"
require "PublicEventObjective"
require "CommunicatorLib"
require "Tooltip"
require "GroupLib"
require "PlayerPathLib"
require "GuildLib"
require "GuildTypeLib"
require "GameLib"

-----------------------------------------------------------------------------------------------
-- ValuePlates Module Definition
-----------------------------------------------------------------------------------------------
local ValuePlates = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

local knNameplatePoolLimit = 200

local tProfiles = 
{
	["Default"] = 
	{
		[1] = false,
		[2] = false,
		[3] = false,
		[4] = false,
		["GeneralSettings"] = 
		{
			["bOcclusion"] = true,
			["bClassificationIcon"] = true,
			["bBigMobPositioning"] = false,
			["bChatBubbleOcclusion"] = false,
			["bRewardIcons"] = true,
			["bQuest"] = true,
			["bPublicEvent"] = true,
			["bMission"] = true,
			["bChallenge"] = true,
			["bFriend"] = true,
			["bRival"] = true,
			["bLevel"] = true,
			["bAggroAlert"] = false,
			["bInvertAggroAlert"] = false,
			["bDispellableAlert"] = false,
			["bDebuffAlert"] = false,
			["bNoShieldMode"] = false,
			["bTargetMarker"] = true,
			["Textures"] = 
			{
				["ShieldBar"] = "VPSprites:Minimalist",
				["HealthBar"] = "VPSprites:Minimalist",
				["CastBar"] = "VPSprites:Minimalist",
				["ResourceBar"] = "VPSprites:Minimalist",
				["TargetMarker"] = "VPSprites:DefaultMarker",
			},
		},
		["TargetSettings"] = 
		{
			["GeneralSection"] = 
			{
				["bShowName"] = true,
				["bTitle"] = true,
				["bAffiliation"] = true,
				["bInCombat"] = false,
			},
			["NameplateSection"] =
			{
				["bShow"] = true,
				["Show"] = "1",
				["bDamaged"] = false,
				["bCasting"] = false,
				["bResourcesDiminished"] = false,
				["bInCombat"] = false,
			},
			["HealthSection"] = 
			{
				["Text"] = "2",
				["Show"] = "1",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["ShieldSection"] = 
			{
				["Text"] = "2",
				["Show"] = "1",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["CastBarSection"] = 
			{
				["bShow"] = true,
				["bName"] = true,
				["bTime"] = true,
				["bIcon"] = false,
				["bInCombat"] = false,
			},
			["InterruptArmorSection"] = 
			{
				["Show"] = "1",
				["bInCombat"] = false,
			},
			["ResourceSection"] = 
			{
				["bShow"] = false,
				["ShowBar"] = "1",
				["bInCombatBar"] = false,
				["Text"] = "2",
				["Show"] = "1",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["ClassColorSection"] = 
			{
				["bShow"] = false,
				["bName"] = false,
				["bHealthBar"] = false,
			},
			["HealthCutoffSection"] = 
			{
				["bCutoff"] = false,
				["HealthCutoffSlider"] = 30,
			},
			["OpacitySection"] = 
			{
				["bOpacity"] = false,
				["OpacitySlider"] = 1,
			},
		},
		["MySettings"] = 
		{
			["GeneralSection"] = 
			{
				["bShowName"] = true,
				["bTitle"] = true,
				["bAffiliation"] = true,
				["bInCombat"] = false,
			},
			["NameplateSection"] =
			{
				["bShow"] = true,
				["Show"] = "1",
				["bDamaged"] = false,
				["bCasting"] = false,
				["bResourcesDiminished"] = false,
				["bInCombat"] = false,
			},
			["HealthSection"] = 
			{
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["ShieldSection"] = 
			{
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["CastBarSection"] = 
			{
				["bShow"] = true,
				["bName"] = true,
				["bTime"] = true,
				["bIcon"] = false,
				["bInCombat"] = false,
			},
			["InterruptArmorSection"] = 
			{
				["Show"] = "2",
				["bInCombat"] = false,
			},
			["ResourceSection"] = 
			{
				["bShow"] = false,
				["ShowBar"] = "1",
				["bInCombatBar"] = false,
				["Text"] = "2",
				["Show"] = "1",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["ClassColorSection"] = 
			{
				["bShow"] = false,
				["bName"] = false,
				["bHealthBar"] = false,
			},
			["HealthCutoffSection"] = 
			{
				["bCutoff"] = false,
				["HealthCutoffSlider"] = 30,
			},
			["OpacitySection"] = 
			{
				["bOpacity"] = false,
				["OpacitySlider"] = 1,
			},
		},
		["NPCSettings"] = 
		{
			["GeneralSection"] = 
			{
				["bShowName"] = true,
				["bTitle"] = true,
				["bAffiliation"] = true,
				["bInCombat"] = false,
			},
			["NameplateSection"] =
			{
				["bShow"] = true,
				["Show"] = "2",
				["bDamaged"] = true,
				["bCasting"] = true,
				["bResourcesDiminished"] = false,
				["bInCombat"] = false,
			},
			["HealthSection"] = 
			{
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["ShieldSection"] = 
			{
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["CastBarSection"] = 
			{
				["bShow"] = true,
				["bName"] = true,
				["bTime"] = true,
				["bIcon"] = false,
				["bInCombat"] = false,
			},
			["InterruptArmorSection"] = 
			{
				["Show"] = "2",
				["bInCombat"] = false,
			},
			["ResourceSection"] = 
			{
				["bShow"] = false,
				["ShowBar"] = "2",
				["bInCombatBar"] = false,
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["HealthCutoffSection"] = 
			{
				["bCutoff"] = false,
				["HealthCutoffSlider"] = 30,
			},
			["OpacitySection"] = 
			{
				["bOpacity"] = false,
				["OpacitySlider"] = 1,
			},
		},
		["PartySettings"] = 
		{
			["GeneralSection"] = 
			{
				["bShowName"] = true,
				["bTitle"] = true,
				["bAffiliation"] = true,
				["bInCombat"] = false,
			},
			["NameplateSection"] =
			{
				["bShow"] = true,
				["Show"] = "2",
				["bDamaged"] = true,
				["bCasting"] = false,
				["bResourcesDiminished"] = false,
				["bInCombat"] = false,
			},
			["HealthSection"] = 
			{
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["ShieldSection"] = 
			{
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["CastBarSection"] = 
			{
				["bShow"] = true,
				["bName"] = true,
				["bTime"] = true,
				["bIcon"] = false,
				["bInCombat"] = false,
			},
			["InterruptArmorSection"] = 
			{
				["Show"] = "1",
				["bInCombat"] = false,
			},
			["ResourceSection"] = 
			{
				["bShow"] = false,
				["ShowBar"] = "2",
				["bInCombatBar"] = false,
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["ClassColorSection"] = 
			{
				["bShow"] = false,
				["bName"] = false,
				["bHealthBar"] = false,
			},
			["HealthCutoffSection"] = 
			{
				["bCutoff"] = false,
				["HealthCutoffSlider"] = 30,
			},
			["OpacitySection"] = 
			{
				["bOpacity"] = false,
				["OpacitySlider"] = 1,
			},
		},
		["PlayerSettings"] = 
		{
			["GeneralSection"] = 
			{
				["bShowName"] = true,
				["bTitle"] = true,
				["bAffiliation"] = true,
				["bInCombat"] = false,
			},
			["NameplateSection"] =
			{
				["bShow"] = true,
				["Show"] = "2",
				["bDamaged"] = true,
				["bCasting"] = true,
				["bResourcesDiminished"] = false,
				["bInCombat"] = false,
			},
			["HealthSection"] = 
			{
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["ShieldSection"] = 
			{
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["CastBarSection"] = 
			{
				["bShow"] = true,
				["bName"] = true,
				["bTime"] = true,
				["bIcon"] = false,
				["bInCombat"] = false,
			},
			["InterruptArmorSection"] = 
			{
				["Show"] = "2",
				["bInCombat"] = false,
			},
			["ResourceSection"] = 
			{
				["bShow"] = false,
				["ShowBar"] = "2",
				["bInCombatBar"] = false,
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["ClassColorSection"] = 
			{
				["bShow"] = false,
				["bName"] = false,
				["bHealthBar"] = false,
			},
			["HealthCutoffSection"] = 
			{
				["bCutoff"] = false,
				["HealthCutoffSlider"] = 30,
			},
			["OpacitySection"] = 
			{
				["bOpacity"] = false,
				["OpacitySlider"] = 1,
			},
		},
		["PetSettings"] = 
		{
			["GeneralSection"] = 
			{
				["bShowName"] = true,
				["bTitle"] = true,
				["bAffiliation"] = true,
				["bInCombat"] = false,
			},
			["NameplateSection"] =
			{
				["bShow"] = true,
				["Show"] = "2",
				["bDamaged"] = true,
				["bCasting"] = true,
				["bResourcesDiminished"] = false,
				["bInCombat"] = false,
			},
			["HealthSection"] = 
			{
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["ShieldSection"] = 
			{
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["CastBarSection"] = 
			{
				["bShow"] = true,
				["bName"] = true,
				["bTime"] = true,
				["bIcon"] = false,
				["bInCombat"] = false,
			},
			["InterruptArmorSection"] = 
			{
				["Show"] = "2",
				["bInCombat"] = false,
			},
			["ResourceSection"] = 
			{
				["bShow"] = false,
				["ShowBar"] = "2",
				["bInCombatBar"] = false,
				["Text"] = "2",
				["Show"] = "2",
				["Format"] = "1",
				["bInCombat"] = false,
			},
			["HealthCutoffSection"] = 
			{
				["bCutoff"] = false,
				["HealthCutoffSlider"] = 30,
			},
			["OpacitySection"] = 
			{
				["bOpacity"] = false,
				["OpacitySlider"] = 1,
			},
		},
		["ColorSettings"] = 
		{
			["General"] = 
			{
				["Background"] = "ff8e8f8d",
				["Backdrop"] = "ff585959",
				["Shield"] = "ff209aa1",
				["Absorb"] = "fffda61f",
				["Vulnerable"] = "ff8b0e91",
				["Cast"] = "ffdfa568",
				["Resource"] = "fffeb308",
			},
			["FriendlyPlayer"] =
			{
				["PvPOff"] = "ff7672ed",
				["PvPOn"] = "ff00ff00",
				["Party"] = "ffaaaaff",
				["Guild"] = "ff00ff00",
				["Friend"] = "ff00ff00",
				["Warplot"] = "ff00ff00",
				["Health"] = "ff000099",
				["HealthCutOff"] = "ff452517",
			},
			["FriendlyNPC"] = 
			{
				["Name"] = "ff02ff08",
				["Health"] = "ff00bb00",
				["HealthCutOff"] = "ffff9408",
			},
			["Class"] = 
			{
				["Warrior"] = "ffde1818",
				["Engineer"] = "ffc08c00",
				["Stalker"] = "ffbc23b8",
				["Medic"] = "ff0f750f",
				["Esper"] = "ff2eccbe",
				["Spellslinger"] = "ff2189aa",
			},
			["HostilePlayer"] =
			{
				["PvPOff"] = "fffe0106",
				["PvPOn"] = "fffe0106",
				["Health"] = "ff8a140a",
				["HealthCutOff"] = "ff452517",
			},
			["HostileNPC"] = 
			{
				["Name"] = "fffe0106",
				["Health"] = "ff8a140a",
				["HealthCutOff"] = "ff452517",
			},
			["NeutralNPC"] = 
			{
				["Name"] = "fffaf22f",
				["Health"] = "ffbbb702",
				["HealthCutOff"] = "ff452517",
			},
		},
	},
}

local settingChoices = 
{	
	["General"] = "GeneralSettings",
	["Target"] = "TargetSettings",
	["Self"] = "MySettings",
	["NPC"] = "NPCSettings",
	["Party"] = "PartySettings",
	["Player"] = "PlayerSettings",
	["Pet"] = "PetSettings",
	["Color"] = "ColorSettings",
}

local nameplateType = 
{
	["Player"] = "PlayerSettings",
	["Simple"] = "NPCSettings",
	["NonPlayer"] = "NPCSettings",
	["Mailbox"] = "NPCSettings",
	["Harvest"] = "NPCSettings",
	["Pet"] = "PetSettings",
	["Party"] = "PartySettings",
	["Target"] = "TargetSettings",
	["Scanner"] = "NPCSettings",
	["Self"] = "MySettings",
	["Mount"] = "PlayerSettings",
	["Turret"] = "NPCSettings",
	["SimpleCollidable"] = "NPCSettings",
	["Plug"] = "NPCSettings",
	["Trap"] = "NPCSettings",
	["Vehicle"] = "NPCSettings",
}

local idToClass = 
{
	[1] = "Warrior",
	[2] = "Engineer",
	[3] = "Esper",
	[4] = "Medic",
	[5] = "Stalker",
	[7] = "Spellslinger",
	[23] = "NPC",
}

local vpBarTextures = 
{
	["Default"] 	= "VPSprites:Default",
	["Glaze"] 		= "VPSprites:Glaze",
	["Otravi"] 		= "VPSprites:Otravi",
	["Smooth"] 		= "VPSprites:Smooth",
	["Minimalist"] 	= "VPSprites:Minimalist",
}

local vpTargetTextures = 
{
	["Default"] 		= "VPSprites:DefaultMarker",
	["Frost"]			= "VPSprites:FrostMarker",
	["Tony"] 			= "VPSprites:TonyMarker",
	["Pappy"] 			= "VPSprites:PappyMarker",
}

local vpClassIcons =
{
	[GameLib.CodeEnumClass.Warrior] 		= "Icon_Windows_UI_CRB_Warrior",
	[GameLib.CodeEnumClass.Engineer] 		= "Icon_Windows_UI_CRB_Engineer",
	[GameLib.CodeEnumClass.Esper]			= "Icon_Windows_UI_CRB_Esper",
	[GameLib.CodeEnumClass.Medic]			= "Icon_Windows_UI_CRB_Medic",
	[GameLib.CodeEnumClass.Stalker] 		= "Icon_Windows_UI_CRB_Stalker",
	[GameLib.CodeEnumClass.Spellslinger]	 = "Icon_Windows_UI_CRB_Spellslinger"
}

local vpMobIcons = 
{
	[Unit.CodeEnumRank.Elite] 			= "VPSprites:Elite",
	[Unit.CodeEnumRank.Superior] 		= "VPSprites:Superior",
	[Unit.CodeEnumRank.Champion]		= "VPSprites:Champion",
	[Unit.CodeEnumRank.Standard]		= "VPSprites:Standard",
	[Unit.CodeEnumRank.Minion] 			= "VPSprites:Minion",
	[Unit.CodeEnumRank.Fodder] 			= "VPSprites:Fodder",
}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ValuePlates:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	o.tPreloadUnits = {}
	o.tUnitNameplates = {}
	o.tWindowNameplates = {}
	o.tWindowPool = {}
	o.tWndSettings = {}	
	o.bAddonRestoredOrLoaded = false
    return o
end

function ValuePlates:Init()
    Apollo.RegisterAddon(self, true, nil, {"Tooltips", "RewardIcons"})
end

function ValuePlates:OnDependencyError(strDependency, strError)
	return true
end
 
-----------------------------------------------------------------------------------------------
-- ValuePlates OnLoad
-----------------------------------------------------------------------------------------------
function ValuePlates:OnLoad()
	Apollo.RegisterEventHandler("UnitCreated", 					"OnPreloadUnitCreated", self)
	
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ValuePlates.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
	Apollo.RegisterTimerHandler("LoadTimer", "OnLoadTimer", self)
	Apollo.CreateTimer("LoadTimer", 5.0, false); Apollo.StopTimer("LoadTimer");	
end

function ValuePlates:OnLoadTimer()
	local unitPlayer = GameLib.GetPlayerUnit()
	if (unitPlayer == nil) then
		Apollo.StarTimer("LoadTimer", 1.0, false)
		return
	end
end

function ValuePlates:OnPreloadUnitCreated(unitNew)
	if (self.tPreloadUnits == nil) then
		self.tPreloadUnits = {}
	end
	
	if (unitNew) then
		self.tPreloadUnits[unitNew:GetId()] = unitNew
	end
end

function ValuePlates:CreateUnitsFromPreload()
	if self.bAddonRestoredOrLoaded then
		self.unitPlayer = GameLib.GetPlayerUnit()
		
		-- Process units created while form was loading
		for idUnit, unitNew in pairs(self.tPreloadUnits) do
			self:OnUnitCreated(unitNew)
		end
		self.tPreloadUnits = nil
	end
	self.bAddonRestoredOrLoaded = true
end


-----------------------------------------------------------------------------------------------
-- ValuePlates OnDocLoaded
-----------------------------------------------------------------------------------------------
function ValuePlates:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ValuePlatesForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		Apollo.RemoveEventHandler("UnitCreated", self)
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("valueplates", "OnValuePlatesOn", self)
		Apollo.RegisterSlashCommand("vp", "OnValuePlatesOn", self)
		Apollo.RegisterSlashCommand("ValuePlates", "OnValuePlatesOn", self)
		Apollo.RegisterSlashCommand("valuePlates", "OnValuePlatesOn", self)
		Apollo.RegisterSlashCommand("Valueplates", "OnValuePlatesOn", self)
		Apollo.RegisterSlashCommand("valueplates reset", "OnValuePlatesReset", self)
		
		Apollo.RegisterEventHandler("UnitCreated", 					"OnUnitCreated", self)
		Apollo.RegisterEventHandler("UnitDestroyed", 				"OnUnitDestroyed", self)
		Apollo.RegisterEventHandler("VarChange_FrameCount", 		"OnFrame", self)
		Apollo.RegisterEventHandler("UnitTextBubbleCreate", 		"OnUnitTextBubbleToggled", self)
		Apollo.RegisterEventHandler("UnitTextBubblesDestroyed", 	"OnUnitTextBubbleToggled", self)
		
		-- Events that cause nameplates colors or settings to swap
		Apollo.RegisterEventHandler("UnitTextBubbleCreate", 		"OnUnitTextBubbleToggled", self)
		Apollo.RegisterEventHandler("UnitTextBubblesDestroyed", 	"OnUnitTextBubbleToggled", self)
		Apollo.RegisterEventHandler("TargetUnitChanged", 			"OnTargetUnitChanged", self)
		Apollo.RegisterEventHandler("UnitEnteredCombat", 			"OnEnteredCombat", self)
		Apollo.RegisterEventHandler("UnitNameChanged", 				"OnUnitNameChanged", self)
		Apollo.RegisterEventHandler("UnitPvpFlagsChanged", 			"OnUnitPvpFlagsChanged", self)
		Apollo.RegisterEventHandler("UnitTitleChanged", 			"OnUnitTitleChanged", self)
		Apollo.RegisterEventHandler("PlayerTitleChange", 			"OnPlayerTitleChanged", self)
		Apollo.RegisterEventHandler("UnitActivationTypeChanged", 	"OnUnitActivationTypeChanged", self)
		Apollo.RegisterEventHandler("UnitLevelChanged", 			"OnUnitLevelChanged", self)
		Apollo.RegisterEventHandler("UnitGuildNameplateChanged", 	"OnUnitGuildNameplateChanged", self)
		Apollo.RegisterEventHandler("UnitMemberOfGuildChange", 		"OnUnitMemberOfGuildChange", self)
		Apollo.RegisterEventHandler("GuildChange", 					"OnGuildChange", self)
		Apollo.RegisterEventHandler("UnitGroupChanged", 			"OnUnitGroupChanged", self)
		Apollo.RegisterEventHandler("FriendshipUpdate",				"OnFriendshipUpdate", self)
		Apollo.RegisterEventHandler("FriendshipRemove",				"OnFriendshipRemove", self)
		Apollo.RegisterEventHandler("FriendshipAccountFriendRemoved",   		"OnFriendshipAccountFriendRemoved", self)
		Apollo.RegisterEventHandler("FriendshipAccountDataUpdate",  			"OnFriendshipAccountDataUpdate", self)
		
		Apollo.RegisterEventHandler("ApplyCCState", 				"OnApplyCCState", self)
		Apollo.RegisterEventHandler("RemoveCCState", 				"OnRemoveCCState", self)
		Apollo.RegisterEventHandler("UnitGibbed",					"OnUnitGibbed", self)
		Apollo.RegisterEventHandler("SpecChanged", 					"OnSpecChanged", self)
		Apollo.RegisterEventHandler("ChangeWorld", 					"OnChangeWorld", self)
		
		Apollo.LoadSprites("VPSprites.xml", "VPSprites")
		
		-- Only update nameplate visiblity 2 times a second instead of every frame
		Apollo.RegisterTimerHandler("VisibilityTimer", "OnVisibilityTimer", self)
		Apollo.CreateTimer("VisibilityTimer", 0.5, true)
		
		local tRewardUpdateEvents = { 	"QuestObjectiveUpdated", "QuestStateChanged", "ChallengeAbandon", "ChallengeLeftArea",
										"ChallengeFailTime", "ChallengeFailArea", "ChallengeActivate", "ChallengeCompleted",
										"ChallengeFailGeneric", "PublicEventObjectiveUpdate", "PublicEventUnitUpdate",
										"PlayerPathMissionUpdate", "FriendshipAdd", "FriendshipRemoved", "FriendshipUpdate" }
		
		for i, str in pairs(tRewardUpdateEvents) do
			Apollo.RegisterEventHandler(str, 						"RequestUpdateAllNameplateRewards", self)
		end

		-- Do additional Addon initialization here
		-- Profile settings
		self.tProfiles = nil
		self.tOptions = nil
		self.currentProfile = self.savedCurrentProfile or "Default"
		self.currentBarChoice = nil
		
		self.bPlayerInCombat = false
		self.bBlinded = false
		self.bRedrawRewardIcons = false
		self.bTexturesChanged = false
		self.bCombatChanged = false
		self.bRedraw = false
		
		self.playerGuild = nil
		self.playerWarParty = nil
		self.unitPlayer = nil
		self.uPreviousTarget = nil
		
		self.currentSetting = nil
		
		self.generalSection = "GeneralSection"
		self.healthSection = "HealthSection"
		self.shieldSection = "ShieldSection"
		self.resourceSection = "ResourceSection"
		self.castBarSection = "CastBarSection"
		self.interruptArmorSection = "InterruptArmorSection"
		self.colorSection = "ColorSection"
		self.healthCutoffSection = "HealthCutoffSection"
		self.sCutoff = "bCutoff"
		self.sHealthCutoffSlider = "HealthCutoffSlider"
		self.opacitySection = "OpacitySection"
		self.sOpacitySlider = "OpacitySlider"
		self.sOpacity = "bOpacity"
		
		self.gSetting = "GeneralSettings"
		self.gSection = "GeneralSection"
		self.tSettinga = "TargetSettings"

		self.oShow = "Show"
		self.oShowBar = "ShowBar"
		self.oText = "Text"
		self.oFormat = "Format"
		self.bIfDamaged = "bDamaged"
		self.bIfCasting = "bCasting"
		self.bIfResources = "bResourcesDiminished"
		
		self.fnPercentages = "HelperFormatPercentage"
		self.fnNumbers = "HelperFormatBigNumber"
		
		-- Set up guild
		for key, guildCurr in pairs(GuildLib.GetGuilds()) do
			local eGuildType = guildCurr:GetType()
			if eGuildType == GuildLib.GuildType_Guild then
				self.playerGuild = guildCurr
			end
			if eGuildType == GuildLib.GuildType_WarParty then
				self.playerWarParty = guildCurr
			end
		end
		
		-- Load saved variables
		if (self.tAnchors ~= nil) then
			self.wndMain:SetAnchorOffsets(unpack(self.tAnchors))
		else
			self:OnValuePlatesReset()
		end
		
		if (self.nSavedDrawDistance ~= nil) then
			self.nDrawDistance = self.nSavedDrawDistance
		else
			self.nDrawDistance = 70.0
		end
		
		if (self.tSavedProfiles ~= nil) then
			tProfiles = self.tSavedProfiles
			self.tOptions = tProfiles[self.currentProfile]
		else
			self.tOptions = tProfiles["Default"]
		end
		
		-- self.tOptions = tProfiles["Default"]

		self.tClassColors = self.tOptions["ColorSettings"]["Class"]
		self:TexturesChanged()
		self:CreateUnitsFromPreload()
	end
end

function ValuePlates:LoadOptions()
	self:CreateGUI()
	
	for setting, t in pairs (self.tOptions) do
		if (type(setting) ~= "number") then
			if (setting == "GeneralSettings") then
				for opt, var in pairs (t) do
					if (opt == "Textures") then
						for name, directory in pairs (var) do
							if (string.find(name, "Target", 1)) then
								for k, v in pairs (vpTargetTextures) do
									if (v == directory) then
										self.tWndSettings[setting]:FindChild(name):FindChild("ComboBox"):SetText(k)
									end
								end
							else
								for k, v in pairs (vpBarTextures) do
									if (v == directory) then
										self.tWndSettings[setting]:FindChild(name):FindChild("ComboBox"):SetText(k)
									end
								end
							end
						end
					else
						self.tWndSettings[setting]:FindChild(opt):SetCheck(var)
					end
				end
			elseif (setting == "ColorSettings") then
				for section, t in pairs (t) do
					for opt, var in pairs (t) do
						self.tWndSettings[setting]:FindChild(section):FindChild(opt):FindChild("Color"):SetText(var)
						self.tWndSettings[setting]:FindChild(section):FindChild(opt):FindChild("Text"):SetTextColor(var)
					end
				end
			else
				for section, t in pairs (t) do
					if (section == "HealthCutoffSection") then
						for opt, var in pairs (t) do
							if (type (var) == "boolean") then
								self.tWndSettings[setting]:FindChild(section):FindChild(opt):SetCheck(var)
							else
								self.tWndSettings[setting]:FindChild(section):FindChild(opt):SetValue(var)
								self.tWndSettings[setting]:FindChild(section):FindChild("Text"):SetText("Cutoff: " .. var .. "%")
							end
						end
					elseif (section == "OpacitySection") then
						for opt, var in pairs (t) do
							if (type (var) == "boolean") then
								self.tWndSettings[setting]:FindChild(section):FindChild(opt):SetCheck(var)
							else
								self.tWndSettings[setting]:FindChild(section):FindChild(opt):SetValue(var * 100)
								self.tWndSettings[setting]:FindChild(section):FindChild("Text"):SetText("Opacity: " .. var * 100 .. "%")
							end
						end
					else
						for opt, var in pairs (t) do
							if (type (var) == "boolean") then
								self.tWndSettings[setting]:FindChild(section):FindChild(opt):SetCheck(var)
							else
								self.tWndSettings[setting]:FindChild(section):FindChild(opt):FindChild(var):SetCheck(true)
							end
						end
					end
				end
			end
		end
	end
	
	self:AddProfileToSettings()
	self:TexturesChanged()
end

-- Used everytime a player swaps or creates a new profile
function ValuePlates:CreateGUI()
	self.wndMain:FindChild("Settings"):DestroyChildren()
	
	self.currentSetting = "GeneralSettings"

	local temp = self.wndMain:FindChild("Options"):GetChildren()
	for k, v in pairs (temp) do
		v:SetCheck(false)
	end
	
	self.wndMain:FindChild("GeneralBtn"):SetCheck(true)

	-- Create windows for settings
	self.wndGeneralSettings = Apollo.LoadForm(self.xmlDoc, "GeneralSettings", self.wndMain:FindChild("Settings"), self)
	self.wndGeneralSettings:FindChild("DrawDistanceSlider"):SetValue(self.nDrawDistance)
	self.wndGeneralSettings:FindChild("DrawDistanceSlider"):GetParent():FindChild("Text"):SetText("Draw Distance: " .. self.nDrawDistance .. "m")
	self.wndGeneralSettings:Show(true)
	self.tWndSettings["GeneralSettings"] = self.wndGeneralSettings
		
	self.wndMySettings = Apollo.LoadForm(self.xmlDoc, "MySettings", self.wndMain:FindChild("Settings"), self)
	self.wndMySettings:Show(false)
	self.tWndSettings["MySettings"] = self.wndMySettings
				
	self.wndTargetSettings = Apollo.LoadForm(self.xmlDoc, "TargetSettings", self.wndMain:FindChild("Settings"), self)
	self.wndTargetSettings:Show(false)
	self.tWndSettings["TargetSettings"] = self.wndTargetSettings
		
	self.wndPlayerSettings = Apollo.LoadForm(self.xmlDoc, "PlayerSettings", self.wndMain:FindChild("Settings"), self)
	self.wndPlayerSettings:Show(false)
	self.tWndSettings["PlayerSettings"] = self.wndPlayerSettings
		
	self.wndNPCSettings = Apollo.LoadForm(self.xmlDoc, "NPCSettings", self.wndMain:FindChild("Settings"), self)
	self.wndNPCSettings:Show(false)
	self.tWndSettings["NPCSettings"] = self.wndNPCSettings
		
	self.wndPartySettings = Apollo.LoadForm(self.xmlDoc, "PartySettings", self.wndMain:FindChild("Settings"), self)
	self.wndPartySettings:Show(false)
	self.tWndSettings["PartySettings"] = self.wndPartySettings
		
	self.wndPetSettings = Apollo.LoadForm(self.xmlDoc, "PetSettings", self.wndMain:FindChild("Settings"), self)
	self.wndPetSettings:Show(false)
	self.tWndSettings["PetSettings"] = self.wndPetSettings
		
	self.wndColorSettings = Apollo.LoadForm(self.xmlDoc, "ColorSettings", self.wndMain:FindChild("Settings"), self)
	self.wndColorSettings:Show(false)
	self.tWndSettings["ColorSettings"] = self.wndColorSettings

	-- Add textures to correct combobox lists
	for name, directory in pairs (vpBarTextures) do
		local newTextureForm = Apollo.LoadForm(self.xmlDoc, "BarTextureForm", self.wndGeneralSettings:FindChild("BarTextureList"), self)
		local newTextureBtn = newTextureForm:FindChild("BarTextureBtn")
		newTextureBtn:SetText(name)
		newTextureBtn:SetData(directory)
		newTextureBtn:ChangeArt(directory .. "Button")
	end
		
	self.wndGeneralSettings:FindChild("BarTextureList"):ArrangeChildrenVert(0)
		
	for name, directory in pairs (vpTargetTextures) do
		local newTextureForm = Apollo.LoadForm(self.xmlDoc, "TargetTextureForm", self.wndGeneralSettings:FindChild("TargetTextureList"), self)
		local newTextureBtn = newTextureForm:FindChild("TargetTextureBtn")
		newTextureBtn:SetText(name)
		newTextureBtn:SetData(directory)
	end
		
	self.wndGeneralSettings:FindChild("TargetTextureList"):ArrangeChildrenVert(0)
end

function ValuePlates:AddProfileToSettings()
	local wndProfile = self.wndGeneralSettings:FindChild("Profiles")
	wndProfile:DestroyChildren()
	
	for name, t in pairs (tProfiles) do
		local btnProfile = Apollo.LoadForm(self.xmlDoc, "ProfileBtn", wndProfile, self)
		btnProfile:SetData(name)
		btnProfile:SetText(name)
		btnProfile:FindChild("1"):SetCheck(t[1])
		btnProfile:FindChild("2"):SetCheck(t[2])
		btnProfile:FindChild("3"):SetCheck(t[3])
		btnProfile:FindChild("4"):SetCheck(t[4])
	end

	wndProfile:ArrangeChildrenVert(0, function(a,b) return a:GetText() < b:GetText() end)
	
	if (wndProfile:FindChildByUserData(self.currentProfile) == nil) then
		wndProfile:FindChildByUserData("Default"):SetCheck(true)
		self.currentProfile = "Default"
		self.tOptions = tProfiles[self.currentProfile]
	else
		wndProfile:FindChildByUserData(self.currentProfile):SetCheck(true)
		self.tOptions = tProfiles[self.currentProfile]
	end
end

function ValuePlates:TexturesChanged()
	self.shieldBarTexture = tProfiles[self.currentProfile]["GeneralSettings"]["Textures"]["ShieldBar"]
	self.healthBarTexture = tProfiles[self.currentProfile]["GeneralSettings"]["Textures"]["HealthBar"]
	self.castBarTexture = tProfiles[self.currentProfile]["GeneralSettings"]["Textures"]["CastBar"]
	self.resourceBarTexture = tProfiles[self.currentProfile]["GeneralSettings"]["Textures"]["ResourceBar"]
	self.targetTexture = tProfiles[self.currentProfile]["GeneralSettings"]["Textures"]["TargetMarker"]
end

-----------------------------------------------------------------------------------------------
-- ValuePlates Functions - Section deals mostly with setting booleans or updating names, levels
-- and so on
-----------------------------------------------------------------------------------------------

function ValuePlates:OnChangeWorld()
	self.uPreviousTarget = nil
	Apollo.CreateTimer("LoadTimer", 1.0, false); Apollo.StopTimer("LoadTimer");	
end

function ValuePlates:OnSpecChanged(newSpecIndex, specError)
	for k, v in pairs (tProfiles) do
		if (v[newSpecIndex]) then
			self:OnProfileBtn(self.wndGeneralSettings:FindChild("Profiles"):FindChildByUserData(k))
			return
		end
	end
end

function ValuePlates:OnTargetUnitChanged(unit)
	self.bRedraw = true
	if (self.uPreviousTarget ~= nil) then
		if (self.uPreviousTarget.unitOwner:IsThePlayer()) then
			self.uPreviousTarget.unitSetting = nameplateType["Self"]
		else
			self.uPreviousTarget.unitSetting = nameplateType[self.uPreviousTarget.unitOwner:GetType()]
		end
		
		self:HelperNameplateSetup(self.uPreviousTarget)
		self:DrawGeneral(self.uPreviousTarget)
		self:DrawNameplate(self.uPreviousTarget)
	end
	
	if (unit ~= nil and self.tUnitNameplates[unit:GetId()] ~= nil) then
		self.uPreviousTarget = self.tUnitNameplates[unit:GetId()]
		self.uPreviousTarget.unitSetting = nameplateType["Target"]
		self:HelperNameplateSetup(self.uPreviousTarget)
		self:DrawGeneral(self.uPreviousTarget)
		self:DrawNameplate(self.uPreviousTarget)
	else
		self.uPreviousTarget = nil
	end
end

function ValuePlates:OnEnteredCombat(unit, bInCombat)
	if (unit == self.unitPlayer) then
		self.bPlayerInCombat = bInCombat
		self.bCombatChanged = true
	end
end

function ValuePlates:OnUnitNameChanged(unitUpdated, strNewName)
	local tNameplate = self.tUnitNameplates[unitUpdated:GetId()]
	if (tNameplate ~= nil) then
		self:DrawName(tNameplate)
	end
end

function ValuePlates:OnUnitPvpFlagsChanged(unitUpdated)
	local tNameplate = self.tUnitNameplates[unitUpdated:GetId()]
	if (tNameplate ~= nil) then
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:OnUnitTitleChanged(unitUpdated)
	local tNameplate = self.tUnitNameplates[unitUpdated:GetId()]
	if (tNameplate ~= nil) then
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:OnPlayerTitleChanged()
	local tNameplate = self.tUnitNameplates[self.unitPlayer:GetId()]
	if (tNameplate ~= nil) then
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:OnUnitLevelChanged(unitUpdated)
	local tNameplate = self.tUnitNameplates[unitUpdated:GetId()]
	if (tNameplate ~= nil) then
		self:DrawLevel(tNameplate)
	end
end

function ValuePlates:OnUnitGuildNameplateChanged(unitUpdated)
	local tNameplate = self.tUnitNameplates[unitUpdated:GetId()]
	if (tNameplate ~= nil) then
		self:OnGuildDirty(unitUpdated, tNameplate)
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:OnUnitMemberOfGuildChange(unitUpdated)
	local tNameplate = self.tUnitNameplates[unitUpdated:GetId()]
	if (tNameplate ~= nil) then
		self:OnGuildDirty(unitUpdated, tNameplate)
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:OnGuildChange()
	self.playerGuild = nil
	self.playerWarParty = nil
	
	for key, guildCurr in pairs(GuildLib.GetGuilds()) do
		if guildCurr:GetType() == GuildLib.GuildType_Guild then
			self.playerGuild = guildCurr
		end
		if guildCurr:GetType() == GuildLib.GuildType_WarParty then
			self.playerWarParty = guildCurr
		end
	end
	
	for idx, tNameplate in pairs(self.tUnitNameplates) do
		self:OnGuildDirty(tNameplate.unitOwner, tNameplate)
		self:SetColors(tNameplate)
	end
end

function ValuePlates:OnGuildDirty(unitOwner, tNameplate)
	if self.playerGuild then
		tNameplate.bIsGuildMember = self.playerGuild:IsUnitMember(unitOwner)
	else
		tNameplate.bIsGuildMember = false
	end

	if self.playerWarParty then
		tNameplate.bIsWarPartyMember = self.playerWarParty:IsUnitMember(unitOwner)
	else
		tNameplate.bIsWarPartyMember = false
	end
end

function ValuePlates:OnUnitGroupChanged(unitUpdated)
	local tNameplate = self.tUnitNameplates[unitUpdated:GetId()]
	
	if (tNameplate == nil) then
		return
	end

	if (unitUpdated:IsThePlayer()) then
		if (unitUpdated:IsInYourGroup()) then
			tNameplate.bIsInGroup = true
		else
			tNameplate.bIsInGroup = false
		end
		self:SetColors(tNameplate)
		return
	end
	
	if (unitUpdated:IsInYourGroup() and not tNameplate.bIsInGroup) then
		tNameplate.unitSetting = nameplateType["Party"]
		tNameplate.bIsInGroup = true
		self:HelperNameplateSetup(tNameplate)
	elseif (not unitUpdated:IsInYourGroup() and tNameplate.bIsInGroup) then
		tNameplate.unitSetting = nameplateType[unitUpdated:GetType()]
		tNameplate.bIsInGroup = false
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:OnFriendshipUpdate(nFriendId)
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		local unitOwner = tNameplate.unitOwner
		tNameplate.bIsFriend = unitOwner:IsFriend()
		tNameplate.bIsAFriend = unitOwner:IsAccountFriend()
		self:SetColors(tNameplate)
	end
end

-- Not working - Removing Friends does NOT update in carbines Unit UserData which means I can't easily grab the new information, must reloadui to update
function ValuePlates:OnFriendshipRemove(nFriendId)
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		local unitOwner = tNameplate.unitOwner
		tNameplate.bIsFriend = unitOwner:IsFriend()
		tNameplate.bIsAFriend = unitOwner:IsAccountFriend()
		self:SetColors(tNameplate)
	end
end

function ValuePlates:OnFriendshipAccountDataUpdate(nId)
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		local unitOwner = tNameplate.unitOwner
		tNameplate.bIsFriend = unitOwner:IsFriend()
		tNameplate.bIsAFriend = unitOwner:IsAccountFriend()
		self:SetColors(tNameplate)
	end
end

-- Not working - Removing Friends does NOT update in carbines Unit UserData which means I can't easily grab the new information, must reloadui to update
function ValuePlates:OnFriendshipAccountFriendRemoved()
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		local unitOwner = tNameplate.unitOwner
		tNameplate.bIsFriend = unitOwner:IsFriend()
		tNameplate.bIsAFriend = unitOwner:IsAccountFriend()
		self:SetColors(tNameplate)
	end
end

-- Something wonky is happening here. Possible that another addon is using a similar function to set targets and they are fighting over which
-- is the real target. And the other function is winning because single clicking the name on a nameplate is not selecting the unit unless you 
-- hold the mouse button down and move your screen (why would this even be the case O.O) or you have sticky targeting enabled. Otherwise it will 
-- select your harder for .5 seconds and deselect it after mouse button is unpressed. 
-- CHECK THIS LATER!!!
function ValuePlates:OnNameplateClick(wndHandler, wndCtrl, eMouseButton)
	local tNameplate = self.tWindowNameplates[wndHandler:GetData()]

	if tNameplate == nil then
		return
	end

	local unitOwner = tNameplate.unitOwner
	if GameLib.GetTargetUnit() ~= unitOwner and eMouseButton == GameLib.CodeEnumInputMouse.Left then
		GameLib.SetTargetUnit(unitOwner)
	end
end

-----------------------------------------------------------------------------------------------
-- ValuePlates Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function ValuePlates:OnConfigure()
	self:OnValuePlatesOn()
end

-- on SlashCommand "/valueplates"
function ValuePlates:OnValuePlatesOn()
	self:LoadOptions()
	self.wndMain:Invoke() -- show the window
end

-- on SlashCommand "/valueplates reset"
function ValuePlates:OnValuePlatesReset()  
	self.wndMain:SetAnchorOffsets(258, 28, 1117, 681)
end

function ValuePlates:OnVisibilityTimer()
	self:UpdateAllNameplateVisibility()
end

function ValuePlates:UpdateAllNameplateVisibility()
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		self:UpdateNameplateVisibility(tNameplate)
		if self.bRedrawRewardIcons then
			self:UpdateRewardIcons(tNameplate)
		end
	end
	
	self.bRedrawRewardIcons = false
end

function ValuePlates:UpdateNameplateVisibility(tNameplate)
	local wndNameplate = tNameplate.wndNameplate
	tNameplate.bOnScreen = wndNameplate:IsOnScreen()
	tNameplate.bOccluded = wndNameplate:IsOccluded()
	local bNewShow = self:CheckDrawDistance(tNameplate) and self:HelperShowNameplate(tNameplate)
	
	--[[
		Got the overall idea but will finish this later. Not an important feature personally. 
		
	local bIsMounted = unitOwner:IsMounted()
	local unitWindow = wndNameplate:GetUnit()
	local unitOwner = tNameplate.unitOwner

	if bIsMounted and unitWindow == unitOwner then
		tNameplate.unitMount = unitOwner
		tNameplate.unitOwner = tNameplate.unitMount:GetUnitMount()
		wndNameplate:SetUnit(tNameplate.unitOwner, 1)
	elseif not bIsMounted and unitWindow ~= unitOwner then
		tNameplate.unitOwner = unitOwner
		tNameplate.unitMount = nil
		wndNameplate:SetUnit(unitOwner, 1)
	end
	--]]
	
	if (tNameplate.unitOwner:IsThePlayer() and tNameplate.unitSetting == "PlayerSettings") then
		tNameplate.unitSetting = "MySettings"
	end
	
	if (bNewShow ~= tNameplate.bShow) then
		wndNameplate:Show(bNewShow)
		tNameplate.bShow = bNewShow
	end
	
	if (bNewShow) then
		self:IsBigMob(tNameplate)
	end
end

function ValuePlates:OnUnitCreated(unit)
	local unitType = unit:GetType()
		--[[
		or unitType == "Turret" 
		or unitType == "SimpleCollidable" 
		or unitType == "Plug" 
		or unitType == "Trap" 
		or unitType == "Vehicle")
		--]] 
		if (unit:GetName() == "Honeywheat") then
			Print (unit:GetType() .. " : " .. tostring(unit:ShouldShowNamePlate()))
		end
	if (not unit:ShouldShowNamePlate() 
		or unitType == "Collectible"
		or unitType == "PinataLoot" 
		or unitType == "Mount"
		or unitType == "Ghost"
		or unitType == "Taxi") then
		return
	end
	
	local idUnit = unit:GetId()
	
	if (self.tUnitNameplates[idUnit] ~= nil and self.tUnitNameplates[idUnit].wndNameplate:IsValid()) then
		return
	end
	
	local wnd = nil
	local wndReferences = nil
	
	if (next(self.tWindowPool) ~= nil) then
		local poolEntry = table.remove(self.tWindowPool)
		wnd = poolEntry[1]
		wndReferences = poolEntry[2]
	end
	
	if (wnd == nil or not wnd:IsValid()) then
		wnd = Apollo.LoadForm(self.xmlDoc, "Default", "InWorldHudStratum", self)
		wndReferences = nil
	end

	wnd:Show(false, true)
	wnd:SetUnit(unit, 1)
	
	local tNameplate =
	{
		unitOwner 			= unit,
		unitMount 			= nil,
		unitSetting 		= nameplateType[unit:GetType()],
		colorSetting 		= nil,
		idUnit 				= unit:GetId(),
		wndNameplate		= wnd,
		bOnScreen 			= wnd:IsOnScreen(),
		bOccluded 			= wnd:IsOccluded(),
		bSpeechBubble 		= false,
		bIsTarget 			= false,
		bIsInGroup			= unit:IsInYourGroup(),
		bIsGuildMember		= false,
		bIsWarPartyMember 	= false,
		bIsFriend			= unit:IsFriend(),
		bIsAFriend			= unit:IsAccountFriend(),
		bIsCluster 			= false,
		bIsCasting			= false,
		bGibbed				= false,
		bShow				= false,
		bIsObjective 		= false,
		bIf 				= false,
		bIsVulnerable 		= false,
		bSpeechBubble		= false,
		bBigMob				= true,
		bShowShield 		= nil,
		nVulnerableTime 	= 0,
		nClass 				= unit:GetClassId(),
		eDisposition		= unit:GetDispositionTo(self.unitPlayer),
		wnd					= wndReferences,
		maxHealth			= 0,
		currentHealth		= 0,
		maxShield			= 0,
		currentShield		= 0,
		maxAbsorb			= 0,
		currentAbsorb		= 0,
	}
	
	self:OnGuildDirty(unit, tNameplate)
	
	if (unit:IsThePlayer()) then
		tNameplate.unitSetting = nameplateType["Self"]
	end

	if (unit:GetShieldCapacityMax() == nil or unit:GetShieldCapacityMax() == 0 or (self.tOptions[self.gSetting]["bNoShieldMode"] and tNameplate.eDisposition == 2)) then
		tNameplate.bShowShield = false
	else
		tNameplate.bShowShield = true
	end
	
	if wndReferences == nil then
		tNameplate.wnd =
		{
			name = wnd:FindChild("Name"),
			affiliation = wnd:FindChild("Affiliation"),
			nameplate = wnd:FindChild("Nameplate"),
			level = wnd:FindChild("Level"),
			shield = wnd:FindChild("Shield"),
			shieldProgress = wnd:FindChild("ShieldBar"),
			shieldText = wnd:FindChild("ShieldText"),
			health = wnd:FindChild("Health"),
			healthProgress = wnd:FindChild("HealthBar"),
			healthText = wnd:FindChild("HealthText"),
			absorbProgress = wnd:FindChild("AbsorbBar"),
			resource = wnd:FindChild("Resource"),
			resourceProgress = wnd:FindChild("ResourceBar"),
			resourceText = wnd:FindChild("ResourceText"),
			castBar = wnd:FindChild("CastBar"),
			castProgress = wnd:FindChild("CastProgress"),
			castName = wnd:FindChild("CastName"),
			castTime = wnd:FindChild("CastTime"),
			rewardIcons = wnd:FindChild("RewardIcons"),
			interruptArmor = wnd:FindChild("InterruptArmor"),
			castIcon = wnd:FindChild("CastIcon"),
			vulnHealth = wnd:FindChild("VulnerableHealth"),
			marker = wnd:FindChild("Marker"),
			aggroAlert = wnd:FindChild("AggroAlert"),
			debuffAlert = wnd:FindChild("DebuffAlert"),
			icon = wnd:FindChild("ClassificationIcon"),
		}
		tNameplate.wnd.interruptArmorBG = tNameplate.wnd.interruptArmor:FindChild("Background")
		tNameplate.wnd.interruptArmorInvuln = tNameplate.wnd.interruptArmor:FindChild("Invuln")
	end
	
	tNameplate.wnd.name:SetData(unit:GetId())
	
	tNameplate.wnd.healthProgress:SetFullSprite(self.healthBarTexture)
	tNameplate.wnd.absorbProgress:SetFullSprite(self.healthBarTexture)
	tNameplate.wnd.shieldProgress:SetFullSprite(self.shieldBarTexture)
	tNameplate.wnd.castProgress:SetFullSprite(self.castBarTexture)
	tNameplate.wnd.resourceProgress:SetFullSprite(self.resourceBarTexture)
	
	self.tUnitNameplates[idUnit] = tNameplate
	self.tWindowNameplates[idUnit] = tNameplate	
	
	self:HelperNameplateSetup(tNameplate)
end

function ValuePlates:OnUnitDestroyed(unit)
	local idUnit = unit:GetId()
	if (self.tUnitNameplates[idUnit] == nil) then
		return
	end

	local tNameplate = self.tUnitNameplates[idUnit]
	local wndNameplate = tNameplate.wndNameplate
	
	self.tWindowNameplates[wndNameplate:GetId()] = nil
	if (#self.tWindowPool < knNameplatePoolLimit) then
		wndNameplate:Show(false, true)
		wndNameplate:SetUnit(nil)
		table.insert(self.tWindowPool, {wndNameplate, tNameplate.wnd})
	else
		wndNameplate:Destroy()
	end
	
	self.tUnitNameplates[idUnit] = nil
end

function ValuePlates:OnUnitGibbed(unitUpdated)
	local tNameplate = self.tUnitNameplates[unitUpdated:GetId()]
	if tNameplate ~= nil then
		tNameplate.bGibbed = true
		self:UpdateNameplateVisibility(tNameplate)
	end
end

function ValuePlates:OnUnitTextBubbleToggled(tUnitArg, strText, nRange)
	local tNameplate = self.tUnitNameplates[tUnitArg:GetId()]
	if tNameplate ~= nil then
		tNameplate.bSpeechBubble = strText ~= nil and strText ~= ""
		self:UpdateNameplateVisibility(tNameplate)
	end
end


function ValuePlates:OnFrame()	
	if (self.unitPlayer ~= nil and self.unitPlayer:IsValid()) then
		for idx, tNameplate in pairs(self.tUnitNameplates) do
			local eDisposition = tNameplate.unitOwner:GetDispositionTo(self.unitPlayer)
			if (tNameplate.eDisposition ~= eDisposition) then
				tNameplate.eDisposition = eDisposition
				self:HelperNameplateSetup(tNameplate)
			end
			
			if (self.bCombatChanged) then
				self:DrawGeneral(tNameplate)
			end
			
			self:DrawNameplate(tNameplate)
		end
	else
		self.unitPlayer = GameLib.GetPlayerUnit()
	end
	self.bRedraw = false
end

function ValuePlates:DrawGeneral(tNameplate)
	local uSetting = tNameplate.unitSetting
	
	if (uSetting == nil) then
		return
	end

	if (not self.tOptions[uSetting][self.gSection]["bInCombat"]) then
		tNameplate.wnd.name:Show(self.tOptions[uSetting][self.gSection]["bShowName"])
		tNameplate.wnd.affiliation:Show(self.tOptions[uSetting][self.gSection]["bAffiliation"])
		return
	else
		if (self.bPlayerInCombat) then
			tNameplate.wnd.name:Show(self.tOptions[uSetting][self.gSection]["bShowName"])
			tNameplate.wnd.affiliation:Show(self.tOptions[uSetting][self.gSection]["bAffiliation"])
		else
			tNameplate.wnd.name:Show(false)
			tNameplate.wnd.affiliation:Show(false)
		end
	end
	
	if (self.bCombatChanged) then
		self.bCombatChanged = false
	end
end

function ValuePlates:DrawNameplate(tNameplate)
	if (not tNameplate.bShow or not self.tOptions[tNameplate.unitSetting]["NameplateSection"]["bShow"]) then
		return
	end
	
	if (self.tOptions[tNameplate.unitSetting]["NameplateSection"]["bInCombat"]) then
		if (not self.bPlayerInCombat) then
			tNameplate.wnd.nameplate:Show(false)
			return
		else
			tNameplate.wnd.nameplate:Show(not tNameplate.bIf)
		end
	end
	
	local wndNameplate = tNameplate.wndNameplate
	local unitOwner = tNameplate.unitOwner

	if (tNameplate.bIf) then
		self:IfCheck(tNameplate)
	end
	
	tNameplate.bIsCasting = unitOwner:ShouldShowCastBar()
	
	-- *Hide nameplate if unit is dead
	if (tNameplate.wnd.nameplate:IsShown()) then
		self:DrawHealth(tNameplate)
		self:DrawShield(tNameplate)
		self:DrawAbsorb(tNameplate)
		self:DrawResource(tNameplate)
		self:DrawCastBar(tNameplate)
		self:DrawInterruptArmor(tNameplate)
		self:DrawVulnerable(tNameplate)
		self:DrawRewards(tNameplate)
	end
	
	self:DrawThreatAlert(tNameplate)
	self:DrawDebuffAlert(tNameplate)
end

-- If statement function
function ValuePlates:IfCheck(tNameplate)
	local unitOwner = tNameplate.unitOwner
	local unitSetting = tNameplate.unitSetting
	local section = "NameplateSection"
	local bShow = false
	
	-- If damage checked
	if (self.tOptions[tNameplate.unitSetting][section][self.bIfDamaged] and not bShow) then
		if (unitOwner:GetHealth() ~= unitOwner:GetMaxHealth() or unitOwner:GetShieldCapacity() ~= unitOwner:GetShieldCapacityMax()) then
			bShow = true
		end
	end
	
	-- If casting checked
	if (self.tOptions[tNameplate.unitSetting][section][self.bIfCasting] and not bShow) then
		bShow = unitOwner:ShouldShowCastBar()
	end
	
	-- If resources active checked (Espers or Warrior is resource > 0, Rest of classes is resource < maxResource)
	if (self.tOptions[tNameplate.unitSetting][section][self.bIfResources] and not bShow) then
		local id = unitOwner:GetClassId()
		if (id == GameLib.CodeEnumClass.Esper or id == GameLib.CodeEnumClass.Warrior or id == GameLib.CodeEnumClass.Engineer) then
			bShow = unitOwner:GetResource(1) ~= 0
		else
			if (id == GameLib.CodeEnumClass.Stalker) then
				bShow = unitOwner:GetResource(3) ~= unitOwner:GetMaxResource(3)
			elseif (id == GameLib.CodeEnumClass.Spellslinger) then
				bShow = unitOwner:GetResource(4) ~= unitOwner:GetMaxResource(4)
			else
				bShow = unitOwner:GetResource(1) ~= unitOwner:GetMaxResource(1)
			end
		end
	end
	
	if (bShow) then
		tNameplate.wnd.nameplate:Show(true)
	else
		tNameplate.wnd.nameplate:Show(false)
	end
end

-- Updates everything big mob related
function ValuePlates:IsBigMob(tNameplate)
	if (not self.tOptions[self.gSetting]["bBigMobPositioning"] or tNameplate.eDisposition == 2) then
		return
	end
	
	local unitOwner = tNameplate.unitOwner
	local wndNameplate = tNameplate.wndNameplate
	
	if unitOwner == nil then
		return
	end
	
	local unitAnchor = unitOwner:GetOverheadAnchor()
	
	if (unitAnchor ~= nil) then
		if (unitAnchor.y - 30 < 0) then
			tNameplate.bBigMob = true
			wndNameplate:SetUnit(unitOwner, 0)
		elseif (tNameplate.bBigMob) then
			tNameplate.bBigMob = false
			wndNameplate:SetUnit(unitOwner, 1)
		end
	end
end

-- Updates everything threat related
function ValuePlates:DrawThreatAlert(tNameplate)
	local bAggro = self.tOptions[self.gSetting]["bAggroAlert"] or self.tOptions[self.gSetting]["bInvertAggroAlert"]
	
	if (not bAggro) then
		return
	end
	
	local unitOwner = tNameplate.unitOwner
	local wndAlert = tNameplate.wnd.aggroAlert
	
	if (tNameplate.eDisposition ~= 0 or not unitOwner:IsInCombat()) then
		if (wndAlert:IsShown()) then
			wndAlert:Show(false)
		end
		return
	end
	
	local bAggroAlert = false
	local bInvertAlert = false
	
	if (self.tOptions[self.gSetting]["bAggroAlert"]) then
		bAggroAlert = true
	elseif (self.tOptions[self.gSetting]["bInvertAggroAlert"]) then
		bInvertAlert = true
	end		
	
	if (unitOwner:GetTarget() == self.unitPlayer and bAggroAlert) then
		wndAlert:SetBGColor("DispositionHostile")
		wndAlert:Show(true)
	elseif (unitOwner:GetTarget() ~= self.unitPlayer and bInvertAlert) then
		wndAlert:SetBGColor("DispositionHostile")
		wndAlert:Show(true)
	elseif (wndAlert:IsShown()) then
		wndAlert:Show(false)
	end
end

-- Updates everything debuff related
function ValuePlates:DrawDebuffAlert(tNameplate)
	local bDebuff = self.tOptions[self.gSetting]["bDebuffAlert"] or self.tOptions[self.gSetting]["bDispellableAlert"]
	if (not bDebuff) then
		return
	end
	
	local unitOwner = tNameplate.unitOwner
	local wndAlert = tNameplate.wnd.debuffAlert
	local bDebuffAlert = false
	local bDispelAlert = false
	local unitDebuffs = unitOwner:GetBuffs().arHarmful

	if (tNameplate.eDisposition ~= 2 or next(unitDebuffs) == nil) then
		if (wndAlert:IsShown()) then
			wndAlert:Show(false)
		end
		return
	end
	
	if (self.tOptions[self.gSetting]["bDebuffAlert"] and unitDebuffs ~= nil) then
		bDebuffAlert = true
	elseif (self.tOptions[self.gSetting]["bDispellableAlert"] and unitDebuffs  ~= nil) then
		for k, debuff in pairs (unitDebuffs) do
			if (debuff["splEffect"]:GetClass() == 38) then
				bDispelAlert = true
				break
			end
		end
	end		

	if (bDebuffAlert) then
		wndAlert:SetBGColor("xkcdAmethyst")
		wndAlert:Show(true)
	elseif (bDispelAlert) then
		wndAlert:SetBGColor("magenta")
		wndAlert:Show(true)
	elseif (wndAlert:IsShown()) then
		wndAlert:Show(false)
	end
end

-- Update everything name related
function ValuePlates:DrawName(tNameplate, bTitle)
	local name = nil
	if (bTitle) then
		name = tNameplate.unitOwner:GetTitleOrName()
	else
		name = tNameplate.unitOwner:GetName()
	end
	
	local wndName = tNameplate.wnd.name
	
	-- Return if name is the same
	if (wndName:GetText() == name) then
		return
	end
	
	tNameplate.wnd.name:SetText(name)
end

-- Update everything guild / faction related
function ValuePlates:DrawGuildOrAffiliation(tNameplate)
	local unit = tNameplate.unitOwner
	local unitType = tNameplate.unitOwner:GetType()
	
	if (unitType == "Player") then
		-- If player then show guild -> circle -> faction / affiliation
		tNameplate.wnd.affiliation:SetText(unit:GetGuildName() or "")
	elseif (unitType == "NonPlayer") then
		-- If NPC then show faction / affiliation
		tNameplate.wnd.affiliation:SetTextRaw(unit:GetAffiliationName() or "")
	else
		-- If neither player or NPC then no title -> return
		tNameplate.wnd.affiliation:SetText("")
	end
	
	if (tNameplate.wnd.affiliation:GetText() == "") then
		tNameplate.wnd.affiliation:Show(false)
		tNameplate.wnd.name:SetAnchorOffsets(-92, -74, 92, -59)
	end
end

-- Update everything level related
function ValuePlates:DrawLevel(tNameplate)
	tNameplate.wnd.level:SetText(tNameplate.unitOwner:GetLevel() or "")
end

-- Update everything classification icon related
function ValuePlates:DrawClassificationIcon(tNameplate, bShowName)
	local unitOwner = tNameplate.unitOwner
	local wndIcon = tNameplate.wnd.icon
	local eRank = unitOwner:GetRank()
	
	if (unitOwner:GetType() == "Player") then
		wndIcon:SetBGColor(self.tClassColors[idToClass[tNameplate.nClass]])
		wndIcon:SetSprite(vpClassIcons[unitOwner:GetClassId()])
	else
		wndIcon:SetBGColor("UI_WindowBGDefault")
		wndIcon:SetSprite(vpMobIcons[eRank])
	end
end

-- Update everything interrupt armor related
function ValuePlates:DrawInterruptArmor(tNameplate)
	if (not tNameplate.wnd.nameplate:IsShown() or tNameplate.bIsVulnerable) then
		return
	end
	
	local wndNameplate = tNameplate.wndNameplate
	local unitOwner = tNameplate.unitOwner
	local wndInterruptArmor = tNameplate.wnd.interruptArmor
	local wndIABG = tNameplate.wnd.interruptArmorBG
	local wndIAInvuln = tNameplate.wnd.interruptArmorInvuln
	local tSettings = self.tOptions[tNameplate.unitSetting][self.interruptArmorSection]
	local IAValue = unitOwner:GetInterruptArmorValue()
	
	-- show settings
	if (tSettings["bInCombat"] and not self.bPlayerInCombat) then
		wndInterruptArmor:Show(false)
	else
		if (tSettings[self.oShow] == "1" and IAValue ~= 0) then
			wndInterruptArmor:Show(true) -- always
		elseif (tSettings[self.oShow] == "2" and IAValue ~= 0) then
			wndInterruptArmor:Show(tNameplate.bIsCasting) -- if casting
		elseif (tSettings[self.oShow] == "3") then
			wndInterruptArmor:Show(false) -- never
			return
		else
			wndInterruptArmor:Show(false) -- never
			return
		end
	end

	if (unitOwner:GetInterruptArmorMax() == -1) then
		wndIAInvuln:Show(true)
		wndIABG:Show(false)
		wndInterruptArmor:SetText("")
	else
		wndIAInvuln:Show(false)
		wndIABG:Show(true)
		wndInterruptArmor:SetText(IAValue)
	end
end

-- Update everything vulnerable related
function ValuePlates:DrawVulnerable(tNameplate)
	local wndNameplate = tNameplate.wndNameplate
	local unitOwner = tNameplate.unitOwner
	local wndInterruptArmor = tNameplate.wnd.interruptArmor
	local wndVulnHealth = tNameplate.wnd.vulnHealth
	local wndIABG = tNameplate.wnd.interruptArmorBG
	local wndIAInvuln = tNameplate.wnd.interruptArmorInvuln

	local nVulnerable = unitOwner:GetCCStateTimeRemaining(Unit.CodeEnumCCState.Vulnerability)
	if (nVulnerable == nil) then
		wndVulnHealth:Show(false)
		wndInterruptArmor:SetTextColor(self.tOptions["ColorSettings"]["General"]["Cast"])
		wndIABG:SetBGColor(self.tOptions["ColorSettings"]["General"]["Cast"])
		tNameplate.bIsVulnerable = false
	elseif (nVulnerable == 0 and nVulnerable ~= tNameplate.nVulnerableTime) then
		tNameplate.nVulnerableTime = 0
		wndVulnHealth:Show(false)
		wndInterruptArmor:SetTextColor(self.tOptions["ColorSettings"]["General"]["Cast"])
		wndIABG:SetBGColor(self.tOptions["ColorSettings"]["General"]["Cast"])
		tNameplate.bIsVulnerable = false
	elseif (nVulnerable ~= 0 and nVulnerable > tNameplate.nVulnerableTime) then
		tNameplate.nVulnerableTime = nVulnerable
		wndVulnHealth:Show(true)
		wndInterruptArmor:Show(true)
		wndInterruptArmor:SetTextColor(self.tOptions["ColorSettings"]["General"]["Vulnerable"])
		wndIABG:SetBGColor(self.tOptions["ColorSettings"]["General"]["Vulnerable"])
		tNameplate.bIsVulnerable = true
	elseif (nVulnerable ~= 0 and nVulnerable < tNameplate.nVulnerableTime) then
		local maxHealth = tNameplate.maxHealth
		local currentHealth = tNameplate.currentHealth
		local nVulnerableTime = tNameplate.nVulnerableTime
		wndVulnHealth:SetMax(maxHealth)
		wndVulnHealth:SetProgress(currentHealth * (nVulnerable / nVulnerableTime))
		wndInterruptArmor:SetText(string.format("%.1f", nVulnerable))
		tNameplate.bIsVulnerable = true
	end
end

function ValuePlates:RequestUpdateAllNameplateRewards()
	self.bRedrawRewardIcons = true
end

-- Update everything reward icon related
function ValuePlates:UpdateRewardIcons(tNameplate)	
	local tFlags = 
	{
		bVert = false,
		bHideQuests = not self.tOptions["GeneralSettings"]["bQuest"],
		bHideChallenges = not self.tOptions["GeneralSettings"]["bChallenge"],
		bHideMissions = not self.tOptions["GeneralSettings"]["bMission"],
		bHidePublicEvents = not self.tOptions["GeneralSettings"]["bPublicEvent"],
		bHideRivals = not self.tOptions["GeneralSettings"]["bRival"],
		bHideFriends = not self.tOptions["GeneralSettings"]["bFriend"],
	}

	if (RewardIcons ~= nil and RewardIcons.GetUnitRewardIconsForm ~= nil) then
		RewardIcons.GetUnitRewardIconsForm(tNameplate.wnd.rewardIcons, tNameplate.unitOwner, tFlags)
		self:DrawRewards(tNameplate)
		tNameplate.wnd.rewardIcons:Show(true)
	else
		tNameplate.wnd.rewardIcons:Show(false)
	end
end

function ValuePlates:DrawRewards(tNameplate)
	local tRewardsData = tNameplate.wnd.rewardIcons:GetData()
	
	if (tNameplate.wnd.rewardIcons:IsShown() and tRewardsData ~= nil and tRewardsData.nIcons ~= nil and tRewardsData.nIcons > 0) then
		local strName = tNameplate.wnd.name:GetText()
		local nNameWidth = Apollo.GetTextWidth("CRB_Interface9_BBO", strName)
		local nHalfNameWidth = nNameWidth / 2

		local wndnameRewardContainer = tNameplate.wnd.rewardIcons
		local rLeft, rTop, rRight, rBottom = wndnameRewardContainer:GetAnchorOffsets()
		local nLeft, nTop, nRight, nBottom = tNameplate.wnd.name:GetAnchorOffsets()
		wndnameRewardContainer:SetAnchorOffsets(nHalfNameWidth, nTop - 5, nHalfNameWidth + wndnameRewardContainer:ArrangeChildrenHorz(0), nBottom + 1)
	end
end

-- Update everything shield related
function ValuePlates:DrawShield(tNameplate)
	local wndNameplate = tNameplate.wndNameplate
	local unitOwner = tNameplate.unitOwner
	local wndShield = tNameplate.wnd.shield
	local wndShieldBar = tNameplate.wnd.shieldProgress
	local wndShieldText = tNameplate.wnd.shieldText
	local tSettings = self.tOptions[tNameplate.unitSetting][self.shieldSection]
	
	local currentShield = unitOwner:GetShieldCapacity()
	local maxShield = unitOwner:GetShieldCapacityMax()

	if (maxShield == nil or maxShield == 0 or (self.tOptions[self.gSetting]["bNoShieldMode"] and tNameplate.eDisposition == 2)) then
		tNameplate.bShowShield = false
	else
		tNameplate.bShowShield = true
	end
	
	if (wndShield:IsShown() ~= tNameplate.bShowShield) then
		self:HelperNameplateSetup(tNameplate)
	end
	
	if (unitOwner:GetType() == "Simple" or currentShield == nil) then
		wndShieldText:SetText("")
		return
	end

	if (tNameplate.currentShield == currentShield and not self.bRedraw) then
		wndShieldBar:SetProgress(currentShield)
		wndShieldBar:SetMax(maxShield)
		return
	end
	
	tNameplate.currentShield = currentShield
	
	-- show settings
	if (tSettings["bInCombat"] and not self.bPlayerInCombat) then
		wndShieldText:Show(false)
	else
		if (tSettings[self.oShow] == "1") then
			wndShieldText:Show(true) -- always
		elseif (tSettings[self.oShow] == "2") then
			wndShieldText:Show(currentShield < maxShield) -- if damaged
		elseif (tSettings[self.oShow] == "3") then
			wndShieldText:Show(false) -- never
			return
		end
	end

	if (tSettings[self.oText] == "1") then
		wndShieldText:SetText(self:HelperDoHealthOrShieldBar(tSettings, currentShield, maxShield, self.fnNumbers)) -- display numbers
	elseif (tSettings[self.oText] == "2") then
		wndShieldText:SetText(self:HelperDoHealthOrShieldBar(tSettings, currentShield, maxShield, self.fnPercentages)) -- display percentages
	end
	
	wndShieldBar:SetProgress(currentShield)
	wndShieldBar:SetMax(maxShield)
end

-- Update everything health related
function ValuePlates:DrawHealth(tNameplate)
	local wndNameplate = tNameplate.wndNameplate
	local unitOwner = tNameplate.unitOwner
	local wndHealth = tNameplate.wnd.health
	local wndHealthBar = tNameplate.wnd.healthProgress
	local wndHealthText = tNameplate.wnd.healthText
	local wndAbsorbBar = tNameplate.wnd.absorbProgress
	local tSettings = self.tOptions[tNameplate.unitSetting][self.healthSection]
	
	if (unitOwner:GetType() == "Simple" or unitOwner:GetHealth() == nil) then
		tNameplate.wnd.nameplate:Show(false)
		return
	end

	local currentHealth = unitOwner:GetHealth()
	local maxHealth = unitOwner:GetMaxHealth()
	
	if (tNameplate.currentHealth == currentHealth and not self.bRedraw) then
		wndHealthBar:SetProgress(currentHealth)
		wndHealthBar:SetMax(maxHealth)
		return
	end

	tNameplate.maxHealth = maxHealth
	tNameplate.currentHealth = currentHealth
	
	-- show settings
	if (tSettings["bInCombat"] and not self.bPlayerInCombat) then
		wndHealthText:Show(false)
	else
		if (tSettings[self.oShow] == "1") then
			wndHealthText:Show(true) -- always
		elseif (tSettings[self.oShow] == "2") then
			wndHealthText:Show(currentHealth ~= maxHealth) -- if damaged
		elseif (tSettings[self.oShow] == "3") then
			wndHealthText:Show(false) -- never
			return
		end
	end
	
	if (tNameplate.currentAbsorb == 0) then
		if (tSettings[self.oText] == "1") then
			wndHealthText:SetText(self:HelperDoHealthOrShieldBar(tSettings, currentHealth, maxHealth, self.fnNumbers)) -- display numbers
		elseif (tSettings[self.oText] == "2") then
			wndHealthText:SetText(self:HelperDoHealthOrShieldBar(tSettings, currentHealth, maxHealth, self.fnPercentages)) -- display percentages
		end
	end
	
	wndHealthBar:SetProgress(currentHealth)
	wndHealthBar:SetMax(maxHealth)
	
	if (self.tOptions[tNameplate.unitSetting][self.healthCutoffSection][self.sCutoff]) then
		if ((currentHealth / maxHealth) * 100 <= self.tOptions[tNameplate.unitSetting][self.healthCutoffSection][self.sHealthCutoffSlider]) then
			wndHealthBar:SetBarColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["HealthCutOff"])
		else
			if (unitOwner:IsACharacter())then
				if (self.tOptions[tNameplate.unitSetting]["ClassColorSection"]["bShow"]) then
					if (self.tOptions[tNameplate.unitSetting]["ClassColorSection"]["bHealthBar"]) then
						wndHealthBar:SetBarColor(self.tClassColors[idToClass[tNameplate.nClass]])
					else
						wndHealthBar:SetBarColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Health"])
					end
				else
					wndHealthBar:SetBarColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Health"])
				end
			else	
				wndHealthBar:SetBarColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Health"])
			end
		end
	end
end

function ValuePlates:DrawAbsorb(tNameplate)
	local wndNameplate = tNameplate.wndNameplate
	local unitOwner = tNameplate.unitOwner
	local wndAbsorbBar = tNameplate.wnd.absorbProgress
	local wndHealthText = tNameplate.wnd.healthText
	local tSettings = self.tOptions[tNameplate.unitSetting][self.healthSection]
	
	if (unitOwner:GetAbsorptionMax() == nil) then
		tNameplate.maxAbsorb = 0
		tNameplate.currentAbsorb = 0
		wndAbsorbBar:Show(false)
		return
	end
	
	local currentAbsorb = unitOwner:GetAbsorptionValue()
	local maxAbsorb = unitOwner:GetAbsorptionMax()

	if (tNameplate.currentAbsorb == currentAbsorb and not self.bRedraw) then
		wndAbsorbBar:SetProgress(currentAbsorb)
		wndAbsorbBar:SetMax(maxAbsorb)
		return
	end

	tNameplate.maxAbsorb = maxAbsorb
	tNameplate.currentAbsorb = currentAbsorb
	
	wndAbsorbBar:SetProgress(currentAbsorb)
	wndAbsorbBar:SetMax(maxAbsorb)
	
	wndAbsorbBar:Show(tNameplate.wnd.healthProgress:IsShown())
	
	if (currentAbsorb ~= 0) then
		if (tSettings[self.oText] == "1") then
			wndHealthText:SetText(self:HelperDoHealthOrShieldBar(tSettings, tNameplate.currentHealth, tNameplate.maxHealth, self.fnNumbers) .. " (" .. self:HelperFormatBigNumber(currentAbsorb) .. ")")
		elseif (tSettings[self.oText] == "2") then
			wndHealthText:SetText(self:HelperDoHealthOrShieldBar(tSettings, tNameplate.currentHealth, tNameplate.maxHealth, self.fnPercentages) .. " (" .. self:HelperFormatPercentage(currentAbsorb / maxAbsorb) .. ")")
		end
	else
		if (tSettings[self.oText] == "1") then
			wndHealthText:SetText(self:HelperDoHealthOrShieldBar(tSettings, tNameplate.currentHealth, tNameplate.maxHealth, self.fnNumbers))
		elseif (tSettings[self.oText] == "2") then
			wndHealthText:SetText(self:HelperDoHealthOrShieldBar(tSettings, tNameplate.currentHealth, tNameplate.maxHealth, self.fnPercentages))
		end

	end
end

-- Update everything resource related
function ValuePlates:DrawResource(tNameplate)
	local tSettings = self.tOptions[tNameplate.unitSetting][self.resourceSection]
	
	if (not tSettings["bShow"]) then
		return
	end
	
	local wndResource = tNameplate.wnd.resource
	local wndNameplate = tNameplate.wndNameplate
	local unitOwner = tNameplate.unitOwner
	local wndResourceBar = tNameplate.wnd.resourceProgress
	local wndResourceText = tNameplate.wnd.resourceText
	local id = unitOwner:GetClassId()
	
	local currentResource = nil
	local maxResource = nil
	
	if (id == GameLib.CodeEnumClass.Stalker) then
		maxResource = unitOwner:GetMaxResource(3)
		currentResource = unitOwner:GetResource(3)
	elseif (id == GameLib.CodeEnumClass.Spellslinger) then
		maxResource = unitOwner:GetMaxResource(4)
		currentResource = unitOwner:GetResource(4)
	else
		maxResource = unitOwner:GetMaxResource(1)		
		currentResource = unitOwner:GetResource(1)
	end
	
	if (unitOwner:GetType() == "Simple" or maxResource == 0) then
		wndResource:Show(false)
		return
	end
	
	-- show settings
	if (tSettings["bInCombatBar"] and not self.bPlayerInCombat) then
		wndResource:Show(false)
	else
		if (tSettings[self.oShowBar] == "1") then
			wndResource:Show(true)
		elseif (tSettings[self.oShowBar] == "2") then
			if (id == GameLib.CodeEnumClass.Esper or id == GameLib.CodeEnumClass.Warrior or id == GameLib.CodeEnumClass.Engineer) then
				wndResource:Show(currentResource ~= 0)
			else
				wndResource:Show(currentResource ~= maxResource)
			end
		end
	end
	
	-- show settings
	if (tSettings["bInCombat"] and not self.bPlayerInCombat) then
		wndResourceText:Show(false)
	else
		if (tSettings[self.oShow] == "1") then
			wndResourceText:Show(true)
		elseif (tSettings[self.oShow] == "2") then
			if (id == GameLib.CodeEnumClass.Esper or id == GameLib.CodeEnumClass.Warrior or id == GameLib.CodeEnumClass.Engineer) then
				wndResourceText:Show(currentResource ~= 0)
			else
				wndResourceText:Show(currentResource ~= maxResource)
			end
		elseif (tSettings[self.oShow] == "3") then
			wndResourceText:Show(false)
			return
		end
	end
	
	if (tSettings[self.oText] == "1") then
		wndResourceText:SetText(self:HelperDoHealthOrShieldBar(tSettings, currentResource, maxResource, self.fnNumbers)) -- display numbers
	elseif (tSettings[self.oText] == "2") then
		wndResourceText:SetText(self:HelperDoHealthOrShieldBar(tSettings, currentResource, maxResource, self.fnPercentages)) -- display percentages
	end
	
	wndResourceBar:SetProgress(currentResource)
	wndResourceBar:SetMax(maxResource)
end

-- Update everything cast related
function ValuePlates:DrawCastBar(tNameplate)
	local wndNameplate = tNameplate.wndNameplate
	local unitOwner = tNameplate.unitOwner
	local wndCastBar = tNameplate.wnd.castBar
	local wndCastProgress = tNameplate.wnd.castProgress
	local wndCastTime = tNameplate.wnd.castTime
	local wndCastName = tNameplate.wnd.castName
	local wndCastIcon = tNameplate.wnd.castIcon
	local tSettings = self.tOptions[tNameplate.unitSetting][self.castBarSection]

	if (not tNameplate.bIsCasting or (tSettings["bInCombat"] and not self.bPlayerInCombat)) then
		wndCastBar:Show(false)
		wndCastName:Show(false)
		wndCastTime:Show(false) 
		wndCastIcon:Show(false)
		return	
	end
	
	if (tNameplate.wnd.resource:IsShown()) then
		wndCastBar:SetAnchorOffsets(-60, -24, 60, -19)
	else
		wndCastBar:SetAnchorOffsets(-60, -33, 60, -28)
	end

	local elapsedTime = unitOwner:GetCastElapsed()
	
	if (tSettings["bShow"]) then
		wndCastBar:Show(true)
		wndCastProgress:SetProgress(elapsedTime)
		wndCastProgress:SetMax(unitOwner:GetCastDuration())
	end
	
	if (tSettings["bName"]) then
		wndCastName:Show(true)
		wndCastName:SetText(unitOwner:GetCastName())
	end
	
	if (tSettings["bTime"]) then
		wndCastTime:Show(true)
		wndCastTime:SetText(string.format("%.1fs", elapsedTime / 1000))
	end
	
	--[[
	if (tSettings["bIcon"]) then
		unit:GetSpellMechanicId() -- Returns IsCasting() spell ID maybe?
		wndCastIcon:FindChild("Icon"):SetSprite()
	end
	--]]
end

-- Set default colors
function ValuePlates:SetColors(tNameplate)
	local unitOwner = tNameplate.unitOwner
	local wnd = tNameplate.wnd
	local unitClass = tNameplate.nClass
	local healthBar = wnd.healthProgress
	local absorbBar = wnd.absorbProgress
	local shieldBar = wnd.shieldProgress
	local resourceBar = wnd.resourceProgress
	local vulnBar = wnd.vulnHealth
	local castBar = wnd.castProgress
	local wndInterrupt = wnd.interruptArmor
	
	-- Health bar color
	if (unitOwner:IsACharacter())then
		if (self.tOptions[tNameplate.unitSetting]["ClassColorSection"]["bShow"]) then
			if (self.tOptions[tNameplate.unitSetting]["ClassColorSection"]["bHealthBar"]) then
				healthBar:SetBarColor(self.tClassColors[idToClass[tNameplate.nClass]])
			else
				healthBar:SetBarColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Health"])
			end
		else
			healthBar:SetBarColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Health"])
		end
	else	
		healthBar:SetBarColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Health"])
	end
	
	-- Shield bar color
	shieldBar:SetBarColor(self.tOptions["ColorSettings"]["General"]["Shield"])
	
	-- Absorb bar color
	absorbBar:SetBarColor(self.tOptions["ColorSettings"]["General"]["Absorb"])
	
	-- Vulnerable bar color
	vulnBar:SetBarColor(self.tOptions["ColorSettings"]["General"]["Vulnerable"])
	self:DrawVulnerable(tNameplate)
	
	-- Cast bar color
	castBar:SetBarColor(self.tOptions["ColorSettings"]["General"]["Cast"])
	self:DrawCastBar(tNameplate)
	
	-- Interrupt armor color
	if (not tNameplate.bIsVulnerable) then
		wndInterrupt:SetTextColor(self.tOptions["ColorSettings"]["General"]["Cast"])
	end
	
	-- Resource bar color
	resourceBar:SetBarColor(self.tOptions["ColorSettings"]["General"]["Resource"])
	
	-- Name color (affiliation set to same color as name in the meantime)
	if (unitOwner:IsACharacter()) then
		if (self.tOptions[tNameplate.unitSetting]["ClassColorSection"]["bShow"] and self.tOptions[tNameplate.unitSetting]["ClassColorSection"]["bName"]) then
			wnd.name:SetTextColor(self.tClassColors[idToClass[tNameplate.nClass]])
			wnd.affiliation:SetTextColor(self.tClassColors[idToClass[tNameplate.nClass]])
			return
		end
	
		if (tNameplate.bIsInGroup) then
			wnd.name:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Party"])
			wnd.affiliation:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Party"])
		elseif (tNameplate.bIsFriend or tNameplate.bIsAFriend) then
			wnd.name:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Friend"])
			wnd.affiliation:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Friend"])
		elseif (not unitOwner:IsThePlayer() and tNameplate.bIsGuildMember) then
			wnd.name:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Guild"])
			wnd.affiliation:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Guild"])
		elseif (not unitOwner:IsThePlayer() and tNameplate.bIsWarPartyMember) then
			wnd.name:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Warplot"])
			wnd.affiliation:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Warplot"])
		else
			if (unitOwner:IsPvpFlagged()) then
				wnd.name:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["PvPOn"])
				wnd.affiliation:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["PvPOn"])
			else
				wnd.name:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["PvPOff"])
				wnd.affiliation:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["PvPOff"])
			end
		end
	else
		wnd.name:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Name"])
		wnd.affiliation:SetTextColor(self.tOptions["ColorSettings"][tNameplate.colorSetting]["Name"])
	end
end

-- Set opacity *REMINDER*
function ValuePlates:SetOpacity(tNameplate)
end

-- Check if nameplate should be drawn
function ValuePlates:CheckDrawDistance(tNameplate)
	local unitPlayer = self.unitPlayer
	local unitOwner = tNameplate.unitOwner
	
	if not unitOwner or not unitPlayer then
	    return false
	end

	tPosTarget = unitOwner:GetPosition()
	tPosPlayer = unitPlayer:GetPosition()

	if tPosTarget == nil then
		return
	end
	
	if tPosPlayer == nil then
		return
	end

	local nDeltaX = tPosTarget.x - tPosPlayer.x
	local nDeltaY = tPosTarget.y - tPosPlayer.y
	local nDeltaZ = tPosTarget.z - tPosPlayer.z

	local nDistance = (nDeltaX * nDeltaX) + (nDeltaY * nDeltaY) + (nDeltaZ * nDeltaZ)

	if tNameplate.bIsTarget or tNameplate.bIsCluster then
		bInRange = nDistance < knTargetRange
		return bInRange
	else
		bInRange = nDistance < (self.nDrawDistance * self.nDrawDistance) -- squaring for quick maths
		return bInRange
	end
end

-- Sets up nameplate
function ValuePlates:HelperNameplateSetup(tNameplate)
	if (tNameplate.eDisposition == 3) then
		tNameplate.eDisposition = tNameplate.unitOwner:GetDispositionTo(self.unitPlayer)
		return
	end
	
	self:HelperColorSetup(tNameplate)
	
	local uSetting = tNameplate.unitSetting

	if (uSetting == nil) then
		return
	end

	if (self.tOptions[self.gSetting]["bLevel"]) then
		self:DrawLevel(tNameplate)
		tNameplate.wnd.level:Show(true)
	else	
		tNameplate.wnd.level:Show(false)
	end
	
	if (self.tOptions[self.gSetting]["bClassificationIcon"]) then
		self:DrawClassificationIcon(tNameplate)
		tNameplate.wnd.icon:Show(true)
	else
		tNameplate.wnd.icon:Show(false)
	end
	
	if (tNameplate.wnd.level:IsShown() and tNameplate.wnd.icon:IsShown()) then
		tNameplate.wnd.level:SetAnchorOffsets(-80, -62, -62, -44)
		tNameplate.wnd.icon:SetAnchorOffsets(-75, -48, -61, -34)
	elseif (tNameplate.wnd.level:IsShown() and not tNameplate.wnd.icon:IsShown()) then
		tNameplate.wnd.level:SetAnchorOffsets(-80, -56, -62, -38)
	elseif ((not tNameplate.wnd.level:IsShown() or tNameplate.wnd.level:GetText() == nil) and tNameplate.wnd.icon:IsShown()) then
		tNameplate.wnd.icon:SetAnchorOffsets(-74, -52, -60, -38)
	end

	self:SetColors(tNameplate) 
	
	if (self.tOptions[self.gSetting]["bRewardIcons"])then
		self:UpdateRewardIcons(tNameplate)
		tNameplate.wnd.rewardIcons:Show(true)
	else	
		tNameplate.wnd.rewardIcons:Show(false)
	end
	
	if (self.tOptions[uSetting][self.gSection]["bShowName"]) then
		self:DrawName(tNameplate, self.tOptions[uSetting][self.gSection]["bTitle"])
	end
	
	if (self.tOptions[uSetting][self.gSection]["bAffiliation"]) then
		tNameplate.wnd.name:SetAnchorOffsets(-92, -90, 92, -70)
		self:DrawGuildOrAffiliation(tNameplate)
	else	
		tNameplate.wnd.name:SetAnchorOffsets(-92, -74, 92, -59)
	end
	
	self:DrawGeneral(tNameplate)
	
	if (tNameplate.unitSetting == "TargetSettings" and self.tOptions[self.gSetting]["bTargetMarker"]) then
		tNameplate.wnd.marker:Show(true)
	else
		tNameplate.wnd.marker:Show(false)
	end
	
	tNameplate.wnd.resource:Show(self.tOptions[tNameplate.unitSetting][self.resourceSection]["bShow"])
	
	if (self.tOptions[uSetting]["OpacitySection"]["bOpacity"]) then
		tNameplate.wnd.nameplate:SetOpacity(self.tOptions[uSetting][self.opacitySection][self.sOpacitySlider])
	else
		tNameplate.wnd.nameplate:SetOpacity(1)
	end
	
	-- Set up shield / health positioning here
	if (not tNameplate.bShowShield) then
		tNameplate.wnd.health:SetAnchorOffsets(-60, -57, 60, -35)
	else
		tNameplate.wnd.health:SetAnchorOffsets(-60, -48, 60, -35)
	end
	tNameplate.wnd.shield:Show(tNameplate.bShowShield)
	
	if (not self.tOptions[uSetting]["NameplateSection"]["bShow"]) then
		tNameplate.wnd.nameplate:Show(false)
		tNameplate.bIf = false
	elseif (self.tOptions[uSetting]["NameplateSection"][self.oShow] == "1") then
		tNameplate.bIf = false
		tNameplate.wnd.nameplate:Show(true)
	elseif (self.tOptions[uSetting]["NameplateSection"][self.oShow] == "2") then
		tNameplate.bIf = true
		self:IfCheck(tNameplate)
	end
	
	if (self.bTexturesChanged) then
		tNameplate.wnd.healthProgress:SetFullSprite(self.healthBarTexture)
		tNameplate.wnd.absorbProgress:SetFullSprite(self.healthBarTexture)
		tNameplate.wnd.shieldProgress:SetFullSprite(self.shieldBarTexture)
		tNameplate.wnd.castProgress:SetFullSprite(self.castBarTexture)
		tNameplate.wnd.resourceProgress:SetFullSprite(self.resourceBarTexture)
		tNameplate.wnd.marker:SetSprite(self.targetTexture)
	end
end

-- Determines nameplate color settings
function ValuePlates:HelperColorSetup(tNameplate) 
	local unitOwner = tNameplate.unitOwner
	local eDisposition = tNameplate.eDisposition

	if (eDisposition == 1) then
		tNameplate.colorSetting = "NeutralNPC"
	elseif (eDisposition == 0) then
		if (unitOwner:GetType() == "Player") then
			tNameplate.colorSetting = "HostilePlayer"
		else
			tNameplate.colorSetting = "HostileNPC"
		end
	elseif (eDisposition == 2) then
		if (unitOwner:GetType() == "Player") then
			tNameplate.colorSetting = "FriendlyPlayer"
		else
			tNameplate.colorSetting = "FriendlyNPC"
		end
	end
end

-- Determines whether or not to show nameplate
function ValuePlates:HelperShowNameplate(tNameplate)
	local unitOwner = tNameplate.unitOwner
	local bHiddenUnit = not unitOwner:ShouldShowNamePlate()
	
	if (bHiddenUnit and not tNameplate.unitSetting == "TargetSettings") then
		return false
	end
	
	if (tNameplate.bGibbed or (tNameplate.bSpeechBubble and self.tOptions[self.gSetting]["bChatBubbleOcclusion"])) then
		return false
	end
	
	if ((self.tOptions[self.gSetting]["bOcclusion"] and tNameplate.bOccluded) or not tNameplate.bOnScreen or unitOwner:IsDead()) then
		return false
	end
	
	if (not self.tOptions[tNameplate.unitSetting]["NameplateSection"]["bShow"]) then
		tNameplate.wnd.nameplate:Show(false)
	end

	if (self.tOptions[tNameplate.unitSetting]["NameplateSection"][self.oShow] == "1") then
		return true
	end

	return true
end

-- Help shield and health numbers
function ValuePlates:HelperDoHealthOrShieldBar(tSettings, current, max, functionCall)
	if (tSettings[self.oFormat] == "1") then
		if (functionCall == self.fnNumbers) then
			return (self:HelperFormatBigNumber(current))
		else
			return (self:HelperFormatPercentage(current / max))
		end
	elseif (tSettings[self.oFormat] == "2") then
		if (functionCall == self.fnNumbers) then
			return (self:HelperFormatBigNumber(current) .. " / " .. self:HelperFormatBigNumber(max))
		else
			return (self:HelperFormatPercentage(current / max) .. " / " .. self:HelperFormatPercentage(1))
		end
	elseif (tSettings[self.oFormat] == "3") then
		if (functionCall == self.fnNumbers) then
			return (self:HelperFormatBigNumber(current - max))
		else
			return ("-" .. self:HelperFormatPercentage(1 - (current  / max)))
		end
	end
end

-- Format percentages
function ValuePlates:HelperFormatPercentage(nArg)
	return string.format("%.1f%%", nArg * 100)
end


-- Format numbers
function ValuePlates:HelperFormatBigNumber(nArg)
	if nArg < 1000 then
		strResult = tostring(nArg)
	elseif nArg < 1000000 then
		if math.floor(nArg%1000/100) == 0 then
			strResult = String_GetWeaselString(Apollo.GetString("TargetFrame_ShortNumberWhole"), math.floor(nArg / 1000))
		else
			strResult = String_GetWeaselString(Apollo.GetString("TargetFrame_ShortNumberFloat"), nArg / 1000)
		end
	elseif nArg < 1000000000 then
		if math.floor(nArg%1000000/100000) == 0 then
			strResult = String_GetWeaselString(Apollo.GetString("TargetFrame_MillionsNumberWhole"), math.floor(nArg / 1000000))
		else
			strResult = String_GetWeaselString(Apollo.GetString("TargetFrame_MillionsNumberFloat"), nArg / 1000000)
		end
	elseif nArg < 1000000000000 then
		if math.floor(nArg%1000000/100000) == 0 then
			strResult = String_GetWeaselString(Apollo.GetString("TargetFrame_BillionsNumberWhole"), math.floor(nArg / 1000000))
		else
			strResult = String_GetWeaselString(Apollo.GetString("TargetFrame_BillionsNumberFloat"), nArg / 1000000)
		end
	else
		strResult = tostring(nArg)
	end
	return strResult
end

-- Format seconds
function ValuePlates:HelperFormatSeconds(nArg)
	return strResult
end

function ValuePlates:OnWorldLocationOnScreen(wndHandler, wndControl, bOnScreen)
	local tNameplate = self.tWindowNameplates[wndHandler:GetId()]
	if tNameplate ~= nil then
		tNameplate.bOnScreen = bOnScreen
		self:UpdateNameplateVisibility(tNameplate)
	end
end

function ValuePlates:OnUnitOcclusionChanged(wndHandler, wndControl, bOccluded)
	local tNameplate = self.tWindowNameplates[wndHandler:GetId()]
	if tNameplate ~= nil then
		tNameplate.bOccluded = bOccluded
		self:UpdateNameplateVisibility(tNameplate)
	end
end


-----------------------------------------------------------------------------------------------
-- ValuePlatesForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function ValuePlates:OnOk()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function ValuePlates:OnClose()
	self.wndMain:Close() -- hide the window
end

function ValuePlates:OnColorChange(wndHandler)
	self.tOptions["ColorSettings"][wndHandler:GetParent():GetParent():GetName()][wndHandler:GetParent():GetName()] = wndHandler:GetText()
	wndHandler:GetParent():FindChild("Text"):SetTextColor(wndHandler:GetText())
	
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:OnOptionsBtn(wndHandler, wndControl) 
	for k, window in pairs (self.tWndSettings) do
		window:Show(false)
	end
	
	local setting = wndHandler:GetText()
	
	self.currentSetting = settingChoices[setting]
	
	self.tWndSettings[self.currentSetting]:Show(true)
	self.wndMain:FindChild("Settings"):ArrangeChildrenVert(0)
end

function ValuePlates:OnDebuffCheckboxCheck(wndHandler, wndControl)
	self.tOptions[self.currentSetting]["bDebuffAlert"] = false
	self.tOptions[self.currentSetting]["bDispellableAlert"] = false
	self.tOptions[self.currentSetting][wndHandler:GetName()] = wndHandler:IsChecked()
end

function ValuePlates:OnDebuffCheckboxUncheck(wndHandler, wndControl)
	self.tOptions[self.currentSetting][wndHandler:GetName()] = wndHandler:IsChecked()
end

function ValuePlates:OnAggroCheckboxCheck(wndHandler, wndControl)
	self.tOptions[self.currentSetting]["bAggroAlert"] = false
	self.tOptions[self.currentSetting]["bInvertAggroAlert"] = false
	self.tOptions[self.currentSetting][wndHandler:GetName()] = wndHandler:IsChecked()
end

function ValuePlates:OnAggroCheckboxUncheck(wndHandler, wndControl)
	self.tOptions[self.currentSetting][wndHandler:GetName()] = wndHandler:IsChecked()
end

function ValuePlates:OnButtonCheck(wndHandler, wndControl)
	self.bRedraw = true
	local settingCategory = self.currentSetting
	local settingSection = wndHandler:GetParent():GetParent():GetName()
	local sectionCategory = wndHandler:GetParent():GetName()

	-- Call function for GeneralSettings
	if (settingSection == "GeneralSettings") then
		self:ChangeGeneralSettings(wndHandler)
	end
	
	-- Call function for GeneralSection
	if (settingSection == "GeneralSection") then
		self:ChangeGeneralSection(wndHandler)
	end
	
	-- Call function for NameplateSection
	if (settingSection == "NameplateSection") then
		self:ChangeNameplateSection(wndHandler)
	end

	-- Call function for HealthSection
	if (settingSection == "HealthSection") then
		self:ChangeHealthSection(wndHandler)
	end
	
	-- Call function for ShieldSection
	if (settingSection == "ShieldSection") then
		self:ChangeShieldSection(wndHandler)
	end
	
	-- Call function for CastBarSection
	if (settingSection == "CastBarSection") then
		self:ChangeCastBarSection(wndHandler)
	end
	
	-- Call function for InterruptArmorSection
	if (settingSection == "InterruptArmorSection") then
		self:ChangeInterruptArmorSection(wndHandler)
	end
	
	-- Call function for ResourceSection
	if (settingSection == "ResourceSection") then
		self:ChangeResourceSection(wndHandler)
	end
	
	-- Call function for ClassColorSection
	if (settingSection == "ClassColorSection") then
		self:ChangeClassColorSection(wndHandler)
	end
	
	-- Call function for HealthCutoffSection
	if (settingSection == "HealthCutoffSection") then
		self:ChangeHealthCutoffSection(wndHandler)
	end
	
	if (settingSection == "OpacitySection") then
		self:ChangeOpacitySection(wndHandler)
	end
	
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:ChangeGeneralSettings(wndHandler)
	self.tOptions[self.currentSetting][wndHandler:GetName()] = wndHandler:IsChecked()
end

function ValuePlates:ChangeGeneralSection(wndHandler)
	self.tOptions[self.currentSetting]["GeneralSection"][wndHandler:GetName()] = wndHandler:IsChecked()
end

function ValuePlates:ChangeNameplateSection(wndHandler)
	if (wndHandler:GetName() == "1" or wndHandler:GetName() == "2") then
		self.tOptions[self.currentSetting]["NameplateSection"][wndHandler:GetParent():GetName()] = tostring(wndHandler:GetName())
	else
		self.tOptions[self.currentSetting]["NameplateSection"][wndHandler:GetName()] = wndHandler:IsChecked()
	end
end

function ValuePlates:ChangeHealthSection(wndHandler)
	if (wndHandler:GetName() == "bInCombat") then
		self.tOptions[self.currentSetting]["HealthSection"][wndHandler:GetName()] = wndHandler:IsChecked()
		return
	end
	self.tOptions[self.currentSetting]["HealthSection"][wndHandler:GetParent():GetName()] = tostring(wndHandler:GetName())
end

function ValuePlates:ChangeShieldSection(wndHandler)
	if (wndHandler:GetName() == "bInCombat") then
		self.tOptions[self.currentSetting]["ShieldSection"][wndHandler:GetName()] = wndHandler:IsChecked()
		return
	end
	self.tOptions[self.currentSetting]["ShieldSection"][wndHandler:GetParent():GetName()] = tostring(wndHandler:GetName())
end

function ValuePlates:ChangeCastBarSection(wndHandler)
	self.tOptions[self.currentSetting]["CastBarSection"][wndHandler:GetName()] = wndHandler:IsChecked()
end

function ValuePlates:ChangeInterruptArmorSection(wndHandler)
	if (wndHandler:GetName() == "bInCombat") then
		self.tOptions[self.currentSetting]["InterruptArmorSection"][wndHandler:GetName()] = wndHandler:IsChecked()
		return
	end
	self.tOptions[self.currentSetting]["InterruptArmorSection"][wndHandler:GetParent():GetName()] = tostring(wndHandler:GetName())
end

function ValuePlates:ChangeResourceSection(wndHandler)
	if (wndHandler:GetName() == "bInCombat" or wndHandler:GetName() == "bShow" or wndHandler:GetName() == "bInCombatBar") then
		self.tOptions[self.currentSetting]["ResourceSection"][wndHandler:GetName()] = wndHandler:IsChecked()
	else
		self.tOptions[self.currentSetting]["ResourceSection"][wndHandler:GetParent():GetName()] = tostring(wndHandler:GetName())
	end
end

function ValuePlates:ChangeClassColorSection(wndHandler)
	self.tOptions[self.currentSetting]["ClassColorSection"][wndHandler:GetName()] = wndHandler:IsChecked()
end

function ValuePlates:ChangeHealthCutoffSection(wndHandler)
	self.tOptions[self.currentSetting]["HealthCutoffSection"][wndHandler:GetName()] = wndHandler:IsChecked()
end

function ValuePlates:ChangeOpacitySection(wndHandler)
	self.tOptions[self.currentSetting]["OpacitySection"][wndHandler:GetName()] = wndHandler:IsChecked()
end

function ValuePlates:OnDrawDistanceSlider(wndNameplate, wndHandler, nValue, nOldValue)
	wndHandler:GetParent():FindChild("Text"):SetText("Draw Distance: " .. nValue .. "m")
	self.nDrawDistance = nValue
end

function ValuePlates:OnHealthCutoffSlider(wndNameplate, wndHandler, nValue, nOldValue)
	wndHandler:GetParent():FindChild("Text"):SetText("Cutoff: " .. nValue .. "%")
	self.tOptions[self.currentSetting]["HealthCutoffSection"][wndHandler:GetName()] = nValue
end

function ValuePlates:OnOpacitySlider(wndNameplate, wndHandler, nValue, nOldValue)
	wndHandler:GetParent():FindChild("Text"):SetText("Opacity: " .. nValue .. "%")
	self.tOptions[self.currentSetting]["OpacitySection"][wndHandler:GetName()] = nValue / 100
end

function ValuePlates:OnProfileSave(wndHandler)
	Apollo.LoadForm(self.xmlDoc, "ProfileForm", nil, self)
end

function ValuePlates:OnProfileDelete(wndHandler)
	if (wndHandler:GetParent():FindChildByUserData(self.currentProfile):GetData() == "Default") then
		local errorForm = Apollo.LoadForm(self.xmlDoc, "ErrorForm", nil, self)
		errorForm:FindChild("Text"):SetText("Error:\nYou cannot delete the default profile!")
		return
	end
	
	tProfiles[wndHandler:GetParent():FindChildByUserData(self.currentProfile):GetData()] = nil
	self:LoadOptions()
	
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:OnProfileOkay(wndHandler)
	local profileName = wndHandler:GetParent():FindChild("Name"):GetText()
	local bSet1 = wndHandler:GetParent():FindChild("ActionSetOne"):IsChecked()
	local bSet2 = wndHandler:GetParent():FindChild("ActionSetTwo"):IsChecked()
	local bSet3 = wndHandler:GetParent():FindChild("ActionSetThree"):IsChecked()
	local bSet4 = wndHandler:GetParent():FindChild("ActionSetFour"):IsChecked()
	local errorString = "Error:"
	
	if (tProfiles[profileName] ~= nil) then
		errorString = errorString .. "\nThe name of this profile is already in use!"
	end
	
	for k, t in pairs (tProfiles) do
		if (bSet1 and t[1]) then
			errorString = errorString .. "\nLimited Action Set 1 is already in use by another profile!"
		end
		if (bSet2 and t[2]) then
			errorString = errorString .. "\nLimited Action Set 2 is already in use by another profile!"
		end
		if (bSet3 and t[3]) then
			errorString = errorString .. "\nLimited Action Set 3 is already in use by another profile!"
		end
		if (bSet4 and t[4]) then
			errorString = errorString .. "\nLimited Action Set 4 is already in use by another profile!"
		end
	end
	
	if (errorString ~= "Error:") then
		local errorForm = Apollo.LoadForm(self.xmlDoc, "ErrorForm", nil, self)
		errorForm:FindChild("Text"):SetText(errorString)
		wndHandler:GetParent():Destroy()
		return
	end
	
	tProfiles[profileName] = self:clone(self.tOptions)
	tProfiles[profileName][1] = wndHandler:GetParent():FindChild("ActionSetOne"):IsChecked()
	tProfiles[profileName][2] = wndHandler:GetParent():FindChild("ActionSetTwo"):IsChecked()
	tProfiles[profileName][3] = wndHandler:GetParent():FindChild("ActionSetThree"):IsChecked()
	tProfiles[profileName][4] = wndHandler:GetParent():FindChild("ActionSetFour"):IsChecked()

	self.currentProfile = profileName
	self.tOptions = tProfiles[self.currentProfile]

	wndHandler:GetParent():Destroy()
	
	self.bRedraw = true
	self:LoadOptions()
end

function ValuePlates:OnProfileCancel(wndHandler)
	wndHandler:GetParent():Destroy()
end

function ValuePlates:OnProfileBtn(wndHandler)
	local profileName = wndHandler:GetData()
	self.currentProfile = profileName
	self.tOptions = tProfiles[self.currentProfile]
	self.bRedraw = true
	self:LoadOptions()
	
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:OnActionSetCheck(wndHandler)
	local profileName = wndHandler:GetParent():GetParent():GetData()
	local actionSet = tonumber(wndHandler:GetName())
	local bInUse = false
	
	for k, t in pairs (tProfiles) do
		if (t[actionSet]) then
			bInUse = true
		end
	end
	
	if (bInUse) then
		local errorForm = Apollo.LoadForm(self.xmlDoc, "ErrorForm", nil, self)
		errorForm:FindChild("Text"):SetText("Error:\nThis action set is already in use!\nYou must deselect it from the other profile currently using it.")
		wndHandler:SetCheck(false)
		return
	end
	
	tProfiles[profileName][actionSet] = wndHandler:IsChecked()
end

function ValuePlates:OnActionSetUncheck(wndHandler)
	local profileName = wndHandler:GetParent():GetParent():GetData()
	local actionSet = tonumber(wndHandler:GetName())
	tProfiles[profileName][actionSet] = wndHandler:IsChecked()
end

function ValuePlates:OnErrorOkay(wndHandler)
	wndHandler:GetParent():Destroy()
end

function ValuePlates:clone(t)
	if (type(t) ~= "table") then 
		return t 
	end
	local meta = getmetatable(t)
	local newProfile = {}
	for k, v in pairs (t) do
		if (type(v) == "table") then
			newProfile[k] = self:clone(v)
		else
			newProfile[k] = v
		end
	end
	setmetatable(newProfile, meta)
	return newProfile
end

function ValuePlates:OnBarBtnCheck(wndHandler)
	self.currentBarChoice = wndHandler:GetParent():GetName()
	local nLeft, nTop, nRight, nBottom = wndHandler:GetParent():GetAnchorOffsets()
	self.wndGeneralSettings:FindChild("BarTextureList"):SetAnchorOffsets(11, nBottom, 173, nBottom + 120)
	self.wndGeneralSettings:FindChild("BarTextureList"):Show(true)
end

function ValuePlates:OnBarBtnUncheck(wndHandler)
	self.wndGeneralSettings:FindChild("BarTextureList"):Show(false)
end

function ValuePlates:OnBarTextureListClosed(wndHandler)
	self.wndGeneralSettings:FindChild(self.currentBarChoice):FindChild("ComboBox"):SetCheck(false)
end

function ValuePlates:OnBarTextureBtn(wndHandler)
	self.wndGeneralSettings:FindChild(self.currentBarChoice):FindChild("ComboBox"):SetText(wndHandler:GetText())
	tProfiles[self.currentProfile]["GeneralSettings"]["Textures"][self.currentBarChoice] = wndHandler:GetData()
	self.wndGeneralSettings:FindChild("BarTextureList"):Show(false)
	self:OnBarTextureListClosed(nil)
	self:TexturesChanged()
	self.bTexturesChanged = true
	
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		self:HelperNameplateSetup(tNameplate)
	end
end

function ValuePlates:OnTargetBtnCheck(wndHandler)
	self.currentBarChoice = wndHandler:GetParent():GetName()
	local nLeft, nTop, nRight, nBottom = wndHandler:GetParent():GetAnchorOffsets()
	self.wndGeneralSettings:FindChild("TargetTextureList"):SetAnchorOffsets(11, nBottom, 173, nBottom + 120)
	self.wndGeneralSettings:FindChild("TargetTextureList"):Show(true)
end

function ValuePlates:OnTargetBtnUncheck(wndHandler)
	self.wndGeneralSettings:FindChild("TargetTextureList"):Show(false)
end

function ValuePlates:OnTargetTextureListClosed(wndHandler)
	self.wndGeneralSettings:FindChild(self.currentBarChoice):FindChild("ComboBox"):SetCheck(false)
end

function ValuePlates:OnTargetTextureBtn(wndHandler)
	self.wndGeneralSettings:FindChild(self.currentBarChoice):FindChild("ComboBox"):SetText(wndHandler:GetText())
	tProfiles[self.currentProfile]["GeneralSettings"]["Textures"][self.currentBarChoice] = wndHandler:GetData()
	self.wndGeneralSettings:FindChild("TargetTextureList"):Show(false)
	self:OnTargetTextureListClosed(nil)
	self:TexturesChanged()
	self.bTexturesChanged = true
	
	for idx, tNameplate in pairs (self.tUnitNameplates) do
		self:HelperNameplateSetup(tNameplate)
	end
end

-----------------------------------------------------------------------------------------------
-- ValuePlates Configurations
-----------------------------------------------------------------------------------------------
-- Save
function ValuePlates:OnSave(eLevel)
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character) then
		return
	end

	local tSave = {}
	
	tSave.tAnchors = {self.wndMain:GetAnchorOffsets()}
	tSave.tSavedProfiles = tProfiles
	tSave.savedCurrentProfile = self.currentProfile
	tSave.nSavedDrawDistance = self.nDrawDistance

	
	return tSave
end

-- Restore
function ValuePlates:OnRestore(eLevel, t)
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character) then
		return
	end

	if (t.tAnchors) then
		self.tAnchors = t.tAnchors		
		self.tSavedProfiles = t.tSavedProfiles
		self.savedCurrentProfile = t.savedCurrentProfile
		self.nSavedDrawDistance = t.nSavedDrawDistance	
	end
	
	self:CreateUnitsFromPreload()
end


-----------------------------------------------------------------------------------------------
-- ValuePlates Instance
-----------------------------------------------------------------------------------------------
local ValuePlatesInst = ValuePlates:new()
ValuePlatesInst:Init()
